@@ -1,3 +1,23 @@
+/***************************************************************************
+ *   Copyright (C) 2002~2005 by Yuking                                     *
+ *   yuking_net@sohu.com                                                   *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc.,                                       *
+ *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ ***************************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -5,6 +25,8 @@
 #include "InputWindow.h"
 
 #include <string.h>
+#include "version.h"
+#include <time.h>
 
 #ifdef _USE_XFT
 #include <ft2build.h>
@@ -15,6 +37,8 @@
 #include "ui.h"
 #include "ime.h"
 
+#include "tools.h"
+
 //˳ܵߵ
 #include "next.xpm"
 #include "prev.xpm"
@@ -25,6 +49,7 @@ int             iInputWindowY = INPUTWND_STARTY;
 int             iTempInputWindowX, iTempInputWindowY;	//¼ʱλãڹģʽ
 
 uint            iInputWindowHeight = INPUTWND_HEIGHT;
+uint            iFixedInputWindowWidth = 0;
 uint            iInputWindowWidth = INPUTWND_WIDTH;
 uint            iInputWindowUpWidth = INPUTWND_WIDTH;
 uint            iInputWindowDownWidth = INPUTWND_WIDTH;
@@ -39,22 +64,18 @@ MESSAGE_COLOR   messageColor[MESSAGE_TYPE_COUNT] = {
     {NULL, {0, 0, 0, 0}}
 };
 
-MESSAGE_COLOR   inputWindowLineColor = { NULL, {0, 100 << 8, 200 << 8, 255 << 8} };	//еɫ
-XColor          colorArrow = { 0, 255 << 8, 150 << 8, 255 << 8 };	//ͷɫ
+MESSAGE_COLOR   inputWindowLineColor = { NULL, {0, 255 << 8, 0, 0} };	//еɫ
+XColor          colorArrow = { 0, 255 << 8, 0, 0 };	//ͷɫ
 
-WINDOW_COLOR    inputWindowColor = { NULL, NULL, {0, 240 << 8, 240 << 8, 240 << 8} };
+WINDOW_COLOR    inputWindowColor = { NULL, NULL, {0, 255 << 8, 240 << 8, 255 << 8} };
 MESSAGE_COLOR   cursorColor = { NULL, {0, 92 << 8, 210 << 8, 131 << 8} };
 
 // *************************************************************
 MESSAGE         messageUp[32];	//ϲʾ
-
-// *************************************************************
 uint            uMessageUp = 0;
 
 // *************************************************************
 MESSAGE         messageDown[32];	//²ʾ
-
-// *************************************************************
 uint            uMessageDown = 0;
 
 XImage         *pNext = NULL, *pPrev = NULL;
@@ -63,9 +84,11 @@ Bool            bIsResizingInputWindow = False;	//ڸıߴʱҪػ
 Bool            bShowPrev = False;
 Bool            bShowNext = False;
 Bool            bTrackCursor = True;
+Bool            bCenterInputWindow = True;
+Bool            bShowInputWindowTriggering = True;
 
 int             iCursorPos = 0;
-Bool            bShowCursor = True;
+Bool            bShowCursor = False;
 
 _3D_EFFECT      _3DEffectInputWindow = _3D_LOWER;
 
@@ -83,6 +106,14 @@ extern XFontSet fontSet;
 extern GC       dimGC;
 extern GC       lightGC;
 
+extern Bool     bUseGBKT;
+
+/* ٶ
+extern Bool     bStartRecordType;
+extern Bool     bShowUserSpeed;
+extern time_t   timeStart;
+extern uint     iHZInputed;
+*/
 Bool CreateInputWindow (void)
 {
     XSetWindowAttributes attrib;
@@ -232,14 +263,50 @@ void DisplayMessage (void)
     XClearArea (dpy, inputWindow, 2, 2, iInputWindowWidth - 2, iInputWindowHeight / 2 - 2, False);
     XClearArea (dpy, inputWindow, 2, iInputWindowHeight / 2 + 1, iInputWindowWidth - 2, iInputWindowHeight / 2 - 2, False);
 
-    iInputWindowUpWidth = 0;
+    if (!uMessageUp && !uMessageDown) {
+	bShowCursor = False;
+	uMessageUp = 1;
+	strcpy (messageUp[0].strMsg, "FCITX V");
+	strcat (messageUp[0].strMsg, FCITX_VERSION);
+	messageUp[0].type = MSG_TIPS;
+
+	/* ʾٶ
+	   if (bStartRecordType && bShowUserSpeed) {
+	   double          timePassed;
+
+	   timePassed = difftime (time (NULL), timeStart);
+	   if (((int) timePassed) == 0)
+	   timePassed = 1.0;
+
+	   uMessageDown = 6;
+	   strcpy (messageDown[0].strMsg, "ٶȣ");
+	   messageDown[0].type = MSG_OTHER;
+	   sprintf (messageDown[1].strMsg, "%d", (int) (iHZInputed * 60 / timePassed));
+	   messageDown[1].type = MSG_CODE;
+	   strcpy (messageDown[2].strMsg, "/  ʱ");
+	   messageDown[2].type = MSG_OTHER;
+	   sprintf (messageDown[3].strMsg, "%d", (int) timePassed);
+	   messageDown[3].type = MSG_CODE;
+	   strcpy (messageDown[4].strMsg, "  ");
+	   messageDown[4].type = MSG_OTHER;
+	   sprintf (messageDown[5].strMsg, "%u", iHZInputed);
+	   messageDown[5].type = MSG_CODE;
+	   }
+	   else { */
+	uMessageDown = 1;
+	strcpy (messageDown[0].strMsg, "http://www.fcitx.org");
+	messageDown[0].type = MSG_CODE;
+	//}
+    }
+
+    iInputWindowUpWidth = 2 * INPUTWND_START_POS_UP + 1;
     for (i = 0; i < uMessageUp; i++) {
 #ifdef _USE_XFT
 	p1 = messageUp[i].strMsg;
 	while (*p1) {
 	    p2 = strTemp;
 	    if (isprint (*p1))	//ʹ
-	    	bEn = True;
+		bEn = True;
 	    else {
 		*p2++ = *p1++;
 		*p2++ = *p1++;
@@ -248,7 +315,7 @@ void DisplayMessage (void)
 	    while (*p1) {
 		if (isprint (*p1)) {
 		    if (!bEn)
-		        break;
+			break;
 		    *p2++ = *p1++;
 		}
 		else {
@@ -259,7 +326,7 @@ void DisplayMessage (void)
 		}
 	    }
 	    *p2 = '\0';
-	    
+
 	    iInputWindowUpWidth += StringWidth (strTemp, (bEn) ? xftFontEn : xftFont);
 	}
 #else
@@ -267,21 +334,20 @@ void DisplayMessage (void)
 #endif
     }
 
-    iInputWindowUpWidth += 2 * INPUTWND_START_POS_UP + 1;
     if (bShowPrev)
 	iInputWindowUpWidth += 16;
     else if (bShowNext)
 	iInputWindowUpWidth += 8;
 
-    iInputWindowDownWidth = 0;
+    iInputWindowDownWidth = 2 * INPUTWND_START_POS_DOWN + 1;
     for (i = 0; i < uMessageDown; i++) {
 #ifdef _USE_XFT
 	p1 = messageDown[i].strMsg;
 	while (*p1) {
-	   p2 = strTemp;
-	   if (isprint (*p1))	//ʹ
+	    p2 = strTemp;
+	    if (isprint (*p1))	//ʹ
 		bEn = True;
-	   else {
+	    else {
 		*p2++ = *p1++;
 		*p2++ = *p1++;
 		bEn = False;
@@ -295,8 +361,8 @@ void DisplayMessage (void)
 		else {
 		    if (bEn)
 			break;
-			*p2++ = *p1++;
-			*p2++ = *p1++;
+		    *p2++ = *p1++;
+		    *p2++ = *p1++;
 		}
 	    }
 	    *p2 = '\0';
@@ -307,7 +373,6 @@ void DisplayMessage (void)
 	iInputWindowDownWidth += StringWidth (messageDown[i].strMsg, fontSet);
 #endif
     }
-    iInputWindowDownWidth += 2 * INPUTWND_START_POS_DOWN + 1;
 
     if (iInputWindowUpWidth < iInputWindowDownWidth)
 	iInputWindowWidth = iInputWindowDownWidth;
@@ -316,6 +381,10 @@ void DisplayMessage (void)
 
     if (iInputWindowWidth < INPUTWND_WIDTH)
 	iInputWindowWidth = INPUTWND_WIDTH;
+    if (iFixedInputWindowWidth) {
+	if (iInputWindowWidth < iFixedInputWindowWidth)
+	    iInputWindowWidth = iFixedInputWindowWidth;
+    }
 
     XGetWindowAttributes (dpy, inputWindow, &wa);
     if ((wa.x + iInputWindowWidth) > DisplayWidth (dpy, iScreen))
@@ -330,6 +399,13 @@ void DisplayMessage (void)
 	i = wa.x;
 
     XMoveWindow (dpy, inputWindow, i, wa.y);
+    if (bCenterInputWindow && !bTrackCursor) {
+	iInputWindowX = (DisplayWidth (dpy, iScreen) - iInputWindowWidth) / 2;
+	if (iInputWindowX < 0)
+	    iInputWindowX = 0;
+	XMoveWindow (dpy, inputWindow, iInputWindowX, iInputWindowY);
+    }
+
     XResizeWindow (dpy, inputWindow, iInputWindowWidth, iInputWindowHeight);
 
     DisplayMessageUp ();
@@ -353,6 +429,8 @@ void DisplayMessageUp (void)
     Bool            bEn;
 #endif
 
+    char           *strGBKT;
+
     iPos = INPUTWND_START_POS_UP;
     iChar = iCursorPos;
 
@@ -383,14 +461,25 @@ void DisplayMessageUp (void)
 	    }
 	    *p2 = '\0';
 
-	    iInputWindowUpWidth = StringWidth (strTemp, (bEn) ? xftFontEn : xftFont);
-	    OutputString (inputWindow, (bEn) ? xftFontEn : xftFont, strTemp, iPos, (2 * iInputWindowHeight - 1) / 5, messageColor[messageUp[i].type].color);
+	    strGBKT = bUseGBKT ? ConvertGBKSimple2Tradition (strTemp) : strTemp;
+
+	    iInputWindowUpWidth = StringWidth (strGBKT, (bEn) ? xftFontEn : xftFont);
+	    OutputString (inputWindow, (bEn) ? xftFontEn : xftFont, strGBKT, iPos, (2 * iInputWindowHeight - 1) / 5, messageColor[messageUp[i].type].color);
 	    iPos += iInputWindowUpWidth;
+
+	    if (bUseGBKT)
+		free (strGBKT);
 	}
 #else
-	iInputWindowUpWidth = StringWidth (messageUp[i].strMsg, fontSet);
-	OutputString (inputWindow, fontSet, messageUp[i].strMsg, iPos, (2 * iInputWindowHeight - 1) / 5, messageColor[messageUp[i].type].gc);
+	strGBKT = bUseGBKT ? ConvertGBKSimple2Tradition (messageUp[i].strMsg) : messageUp[i].strMsg;
+
+	iInputWindowUpWidth = StringWidth (strGBKT, fontSet);
+	OutputString (inputWindow, fontSet, strGBKT, iPos, (2 * iInputWindowHeight - 1) / 5, messageColor[messageUp[i].type].gc);
 	iPos += iInputWindowUpWidth;
+
+	if (bUseGBKT)
+	    free (strGBKT);
+
 #endif
 
 	if (bShowCursor && iChar) {
@@ -455,18 +544,21 @@ void DisplayMessageDown (void)
     Bool            bEn;
 #endif
 
+    char           *strGBKT;
+
     iPos = INPUTWND_START_POS_DOWN;
     for (i = 0; i < uMessageDown; i++) {
 	//iInputWindowDownWidthΪһʱ
 
 #ifdef _USE_XFT
 	p1 = messageDown[i].strMsg;
+
 	while (*p1) {
 	    p2 = strTemp;
 	    if (isprint (*p1))	//ʹ
 		bEn = True;
 	    else {
-	        *p2++ = *p1++;
+		*p2++ = *p1++;
 		*p2++ = *p1++;
 		bEn = False;
 	    }
@@ -485,18 +577,28 @@ void DisplayMessageDown (void)
 	    }
 	    *p2 = '\0';
 
-	    iInputWindowDownWidth = StringWidth (strTemp, (bEn) ? xftFontEn : xftFont);
-	    OutputString (inputWindow, (bEn) ? xftFontEn : xftFont, strTemp, iPos, (9 * iInputWindowHeight - 12) / 10, messageColor[messageDown[i].type].color);
+	    strGBKT = bUseGBKT ? ConvertGBKSimple2Tradition (strTemp) : strTemp;
+
+	    iInputWindowDownWidth = StringWidth (strGBKT, (bEn) ? xftFontEn : xftFont);
+	    OutputString (inputWindow, (bEn) ? xftFontEn : xftFont, strGBKT, iPos, (9 * iInputWindowHeight - 12) / 10, messageColor[messageDown[i].type].color);
 	    iPos += iInputWindowDownWidth;
+
+	    if (bUseGBKT)
+		free (strGBKT);
 	}
 
 	/*iInputWindowDownWidth = StringWidth (messageDown[i].strMsg, xftFont);
 	   OutputString (inputWindow, xftFont, messageDown[i].strMsg, iPos, (9 * iInputWindowHeight - 12) / 10, messageColor[messageDown[i].type].color);
 	   iPos += iInputWindowDownWidth; */
 #else
-	iInputWindowDownWidth = StringWidth (messageDown[i].strMsg, fontSet);
-	OutputString (inputWindow, fontSet, messageDown[i].strMsg, iPos, (9 * iInputWindowHeight - 12) / 10, messageColor[messageDown[i].type].gc);
+	strGBKT = bUseGBKT ? ConvertGBKSimple2Tradition (messageDown[i].strMsg) : messageDown[i].strMsg;
+
+	iInputWindowDownWidth = StringWidth (strGBKT, fontSet);
+	OutputString (inputWindow, fontSet, strGBKT, iPos, (9 * iInputWindowHeight - 12) / 10, messageColor[messageDown[i].type].gc);
 	iPos += iInputWindowDownWidth;
+
+	if (bUseGBKT)
+	    free (strGBKT);
 #endif
     }
 }
