@@ -1,4 +1,24 @@
+/***************************************************************************
+ *   Copyright (C) 2002~2005 by Yuking                                     *
+ *   yuking_net@sohu.com                                                   *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc.,                                       *
+ *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ ***************************************************************************/
 #include <ctype.h>
+#include <time.h>
 
 #ifdef _USE_XFT
 #include <ft2build.h>
@@ -17,6 +37,8 @@
 #include "table.h"
 #include "tools.h"
 #include "ui.h"
+#include "vk.h"
+#include "QuickPhrase.h"
 
 IM             *im = NULL;
 INT8            iIMCount = 0;
@@ -45,23 +67,28 @@ Bool            bChnPunc = True;	//Ӣıл
 Bool            bUseGBK = False;	//Ƿ֧GBK
 Bool            bIsDoInputOnly = False;	//Ƿֻ뷨
 Bool            bLastIsNumber = False;	//һǲǰ
-Bool            bInCap = False;	//ǲǴڴдӢ״̬
-Bool            bAutoHideInputWindow = True;	//ǷԶ
+INT8            iInCap = 0;	//ǲǴڴдӢ״̬,0--1--´д2--·ֺż
+Bool            bAutoHideInputWindow = False;	//ǷԶ
 Bool            bEngPuncAfterNumber = True;	//ֺǷ(ֻ'.'/','Ч)
-Bool            bPhraseTips = False;
+Bool            bPhraseTips = True;
 INT8            lastIsSingleHZ = 0;
+Bool            bUseGBKT = False;
 
-Bool            bEngAfterSemicolon = False;
+SEMICOLON_TO_DO semicolonToDo = K_SEMICOLON_QUICKPHRASE;
 Bool            bEngAfterCap = True;
+Bool            bConvertPunc = True;
 Bool            bDisablePagingInLegend = True;
 
+Bool            bVK = False;
+
 int             i2ndSelectKey = 0;	//ڶѡѡΪɨ
 int             i3rdSelectKey = 0;	//ѡѡΪɨ
 
 Time            lastKeyPressedTime;
+unsigned int    iTimeInterval = 250;
 
 KEY_RELEASED    keyReleased = KR_OTHER;
-Bool		bDoubleSwitchKey = False;
+Bool            bDoubleSwitchKey = False;
 KEYCODE         switchKey = L_CTRL;
 
 //ȼ
@@ -79,11 +106,25 @@ Bool            bIsInLegend = False;
 
 INT8            iIMIndex = 0;
 Bool            bUsePinyin = True;
-Bool            bUseSP = False;
+Bool            bUseSP = True;
 Bool            bUseQW = True;
 Bool            bUseTable = True;
 
+/* µLumaQQѾҪ֧
 Bool            bLumaQQ = False;
+*/
+Bool            bPointAfterNumber = True;
+
+/* ٶȵĹûʲôôȥ */
+/* ٶ */
+/*
+time_t          timeStart;
+Bool            bStartRecordType;
+Bool            bShowUserSpeed = True;
+uint            iHZInputed = 0;
+*/
+
+/* ************************ */
 
 //++++++++++++++++++++++++++++++++++++++++
 /*
@@ -111,15 +152,21 @@ extern XIMTriggerKey *Trigger_Keys;
 extern Window   mainWindow;
 extern int      iCursorPos;
 
+extern Window   VKWindow;
+extern VKS      vks[];
+extern unsigned char iCurrentVK;
+extern Bool     bVK;
+
 extern int      MAINWND_WIDTH;
 extern Bool     bLocked;
 extern Bool     bCompactMainWindow;
+extern Bool     bShowVK;
 
 extern INT8     iTableChanged;
 extern INT8     iNewPYPhraseCount;
 extern INT8     iOrderCount;
 extern INT8     iNewFreqCount;
-extern INT8     iTableOrderChanged;
+extern INT16    iTableOrderChanged;
 
 extern TABLE   *table;
 extern INT8     iTableCount;
@@ -156,9 +203,9 @@ void ResetInput (void)
 
     bShowPrev = False;
     bShowNext = False;
-
+   
     bIsInLegend = False;
-    bInCap = False;
+    iInCap = 0;
 
     if (IsIM (NAME_OF_PINYIN))
 	bSingleHZMode = False;
@@ -172,32 +219,66 @@ void ResetInput (void)
 void CloseIM (IMForwardEventStruct * call_data)
 {
     XUnmapWindow (dpy, inputWindow);
+    XUnmapWindow (dpy, VKWindow);
     IMPreeditEnd (ims, (XPointer) call_data);
     SetConnectID (call_data->connect_id, IS_CLOSED);
+    bVK = False;
+    SwitchIM (-2);
     DisplayMainWindow ();
 }
 
-void ChangeIMState(IMForwardEventStruct * call_data)
+void ChangeIMState (CARD16 _connect_id)
 {
-    if (ConnectIDGetState (call_data->connect_id) == IS_ENG) {
-	SetConnectID (call_data->connect_id, IS_CHN);
-	DisplayInputWindow ();
-	
-	if ( ConnectIDGetTrackCursor (call_data->connect_id) )
+    if (ConnectIDGetState (_connect_id) == IS_ENG) {
+	SetConnectID (_connect_id, IS_CHN);
+	if (bVK)
+	    DisplayVKWindow ();
+	else
+	    DisplayInputWindow ();
+
+	if (ConnectIDGetTrackCursor (_connect_id))
 	    XMoveWindow (dpy, inputWindow, iTempInputWindowX, iTempInputWindowY);
 	else
 	    XMoveWindow (dpy, inputWindow, iInputWindowX, iInputWindowY);
     }
     else {
-	SetConnectID (call_data->connect_id, IS_ENG);
+	SetConnectID (_connect_id, IS_ENG);
 	ResetInput ();
 	ResetInputWindow ();
 	XUnmapWindow (dpy, inputWindow);
+	XUnmapWindow (dpy, VKWindow);
     }
+
     if (hideMainWindow != HM_HIDE)
 	DisplayMainWindow ();
 }
 
+/*
+ * תstrStringGetеıΪı
+ */
+void ConvertPunc (void)
+{
+    char            strTemp[MAX_USER_INPUT + 1] = "\0";
+    char           *s1, *s2, *pPunc;
+
+    s1 = strTemp;
+    s2 = strStringGet;
+
+    while (*s2) {
+	pPunc = GetPunc (*s2);
+	if (pPunc) {
+	    strcat (s1, pPunc);
+	    s1 += strlen (pPunc);
+	}
+	else
+	    *s1++ = *s2;
+	s2++;
+    }
+    *s2 = '\0';
+
+    strcpy (strStringGet, strTemp);
+}
+
 //FILE           *fd;
 void ProcessKey (IMForwardEventStruct * call_data)
 {
@@ -210,18 +291,14 @@ void ProcessKey (IMForwardEventStruct * call_data)
     int             iKey;
     char           *pstr;
     int             iLen;
- //   IC		   *ic;
 
     kev = (XKeyEvent *) & call_data->event;
     memset (strbuf, 0, STRBUFLEN);
     keyCount = XLookupString (kev, strbuf, STRBUFLEN, &keysym, NULL);
 
     iKeyState = kev->state - (kev->state & KEY_NUMLOCK) - (kev->state & KEY_CAPSLOCK) - (kev->state & KEY_SCROLLLOCK);
-
     iKey = GetKey (keysym, iKeyState, keyCount);
 
-    //printf("COUNT:%d STATE:%d KEYSYM:%d IKEY: %d(%c)\n",keyCount,iKeyState,(char)keysym,iKey,iKey);
-
     if (!iKey)
 	return;
 
@@ -229,11 +306,9 @@ void ProcessKey (IMForwardEventStruct * call_data)
      * xineкѡԶѡе
      * xineÿӲһSHIFTͷ¼
      */
-    /*ic=(IC *)FindIC (call_data->icid);
-    /printf("%d %d %d %d %d %d\n",kev->same_screen,ic->focus_win,ic->client_win,kev->window,kev->root,kev->subwindow); */
     if (kev->same_screen && (kev->keycode == switchKey || kev->keycode == i2ndSelectKey || kev->keycode == i3rdSelectKey))
-    	return;
-    
+	return;
+
     retVal = IRV_TO_PROCESS;
 
     if (call_data->event.type == KeyRelease) {
@@ -251,7 +326,7 @@ void ProcessKey (IMForwardEventStruct * call_data)
 		    CloseIM (call_data);
 	    }
 	    else if (kev->keycode == switchKey && keyReleased == KR_CTRL && !bDoubleSwitchKey) {
-	        ChangeIMState(call_data);
+		ChangeIMState (call_data->connect_id);
 		retVal = IRV_DO_NOTHING;
 	    }
 	    else if (kev->keycode == i2ndSelectKey && keyReleased == KR_2ND_SELECTKEY) {
@@ -270,9 +345,9 @@ void ProcessKey (IMForwardEventStruct * call_data)
 			retVal = IRV_TO_PROCESS;
 		}
 		else {
-			strcpy(strStringGet, " ");
-			uMessageDown = 0;
-			retVal = IRV_GET_CANDWORDS;
+		    strcpy (strStringGet, " ");
+		    uMessageDown = 0;
+		    retVal = IRV_GET_CANDWORDS;
 		}
 	    }
 	    else if (kev->keycode == i3rdSelectKey && keyReleased == KR_3RD_SELECTKEY) {
@@ -289,9 +364,9 @@ void ProcessKey (IMForwardEventStruct * call_data)
 			retVal = IRV_DISPLAY_CANDWORDS;
 		}
 		else {
-			strcpy(strStringGet, "");
-			uMessageDown = 0;
-			retVal = IRV_GET_CANDWORDS;
+		    strcpy (strStringGet, "");
+		    uMessageDown = 0;
+		    retVal = IRV_GET_CANDWORDS;
 		}
 	    }
 	}
@@ -299,13 +374,13 @@ void ProcessKey (IMForwardEventStruct * call_data)
 
     if (retVal == IRV_TO_PROCESS) {
 	if (call_data->event.type == KeyPress) {
-	    if (kev->keycode != switchKey )
+	    if (kev->keycode != switchKey)
 		keyReleased = KR_OTHER;
 	    else {
-	        if ( (keyReleased == KR_CTRL) && (kev->time-lastKeyPressedTime<250) && bDoubleSwitchKey) 
-		    ChangeIMState(call_data);
+		if ((keyReleased == KR_CTRL) && (kev->time - lastKeyPressedTime < iTimeInterval) && bDoubleSwitchKey)
+		    ChangeIMState (call_data->connect_id);
 	    }
-	    
+
 	    lastKeyPressedTime = kev->time;
 	    if (kev->keycode == switchKey) {
 		keyReleased = KR_CTRL;
@@ -316,8 +391,8 @@ void ProcessKey (IMForwardEventStruct * call_data)
 		if (ConnectIDGetState (call_data->connect_id) == IS_ENG) {
 		    SetConnectID (call_data->connect_id, IS_CHN);
 		    DisplayInputWindow ();
-		    
-		    if ( ConnectIDGetTrackCursor (call_data->connect_id) )
+
+		    if (ConnectIDGetTrackCursor (call_data->connect_id))
 			XMoveWindow (dpy, inputWindow, iTempInputWindowX, iTempInputWindowY);
 		    else
 			XMoveWindow (dpy, inputWindow, iInputWindowX, iInputWindowY);
@@ -326,230 +401,273 @@ void ProcessKey (IMForwardEventStruct * call_data)
 			DisplayMainWindow ();
 
 		}
-		else {
-			CloseIM (call_data);
-			retVal = IRV_DO_NOTHING;
-		}
+		else
+		    CloseIM (call_data);
+
+		retVal = IRV_DO_NOTHING;
 	    }
 	}
-	if (retVal == IRV_TO_PROCESS && (ConnectIDGetState (call_data->connect_id) == IS_CHN)) {
+
+	if (retVal == IRV_TO_PROCESS) {
 	    if (call_data->event.type == KeyPress) {
-		if (iKeyState == KEY_NONE) {
-		    if (kev->keycode == i2ndSelectKey)
-			keyReleased = KR_2ND_SELECTKEY;
-		    else if (kev->keycode == i3rdSelectKey)
-			keyReleased = KR_3RD_SELECTKEY;
-		}
-
-		if (iKey == CTRL_LSHIFT || iKey == SHIFT_LCTRL) {
-		    if (bLocked)
-			retVal = IRV_TO_PROCESS;
-		}
-		else {
-		    if (!bInCap && !bCorner) {
-			retVal = im[iIMIndex].DoInput (iKey);
-			if (!IsIM (NAME_OF_PINYIN) && !IsIM (NAME_OF_SHUANGPIN))
-			    iCursorPos = iCodeInputCount;
-		    }
-
-		    if (!bIsDoInputOnly && retVal == IRV_TO_PROCESS) {
-			if (bCorner && (iKey >= 32 && iKey <= 126)) {
-			    //˱ ո ȫǲԣȷ0xa1 0xa1
-			    //ȴ˵ȫǷ0xa3ʼ
-			    //0xa3 0xa0ܻʾ룬˲0xa1 0xa1ķʽ
-			    if (iKey == ' ')
-				sprintf (strStringGet, "%c%c", 0xa1, 0xa1);
-			    else
-				sprintf (strStringGet, "%c%c", 0xa3, 0xa0 + iKey - 32);
-			    retVal = IRV_GET_CANDWORDS;
+		if (ConnectIDGetState (call_data->connect_id) == IS_CHN) {
+		    if (bVK)
+			retVal = DoVKInput (iKey);
+		    else {
+			if (iKeyState == KEY_NONE) {
+			    if (kev->keycode == i2ndSelectKey)
+				keyReleased = KR_2ND_SELECTKEY;
+			    else if (kev->keycode == i3rdSelectKey)
+				keyReleased = KR_3RD_SELECTKEY;
 			}
-			else if (iKey >= 'A' && iKey <= 'Z' && bEngAfterCap && !(kev->state & KEY_CAPSLOCK)) {
-			    bInCap = True;
-			    if (!bIsInLegend && iCandWordCount) {
-				pstr = im[iIMIndex].GetCandWord (0);
-				iCandWordCount = 0;
-				if (pstr) {
-				    strcpy (strStringGet, pstr);
-				    SendHZtoClient (call_data, strStringGet);
-				    iCodeInputCount = 0;
-				}
-			    }
-			}
-			else if (iKey == ';' && bEngAfterSemicolon && !iCodeInputCount)
-			    bInCap = True;
-			else if (IsHotKey (iKey, hkCorner))
-			    retVal = ChangeCorner ();
-			else if (IsHotKey (iKey, hkPunc))
-			    retVal = ChangePunc ();
-			else if (IsHotKey (iKey, hkGBK))
-			    retVal = ChangeGBK ();
-			else if (IsHotKey (iKey, hkPrevPage))
-			    retVal = im[iIMIndex].GetCandWords (SM_PREV);
-			else if (IsHotKey (iKey, hkNextPage))
-			    retVal = im[iIMIndex].GetCandWords (SM_NEXT);
-			else if (IsHotKey (iKey, hkLegend))
-			    retVal = ChangeLegend ();
-			else if (IsHotKey (iKey, hkTrack))
-			    retVal = ChangeTrack ();
 
-			if (retVal == IRV_TO_PROCESS) {
-			    if (bInCap) {
-				if (iKey == ' ') {
-				    if (iCodeInputCount == 0)
-					strcpy (strStringGet, "");
-				    else
-					strcpy(strStringGet,strCodeInput);
-				    retVal = IRV_ENG;
-				    bInCap = False;
-				}
-				else if (isprint (iKey) && iKey < 128) {
-				    if (iCodeInputCount == MAX_USER_INPUT)
-					retVal = IRV_DO_NOTHING;
-				    else {
-					if (!bEngAfterSemicolon || !(bEngAfterSemicolon && (iCodeInputCount == 0 && iKey == ';'))) {
-					    strCodeInput[iCodeInputCount++] = iKey;
-					    strCodeInput[iCodeInputCount] = '\0';
-					    bShowCursor = True;
-					    iCursorPos = iCodeInputCount;
-					}
-					retVal = IRV_DISPLAY_MESSAGE;
-				    }
-				}
-				else if (iKey == (XK_BackSpace & 0x00FF)
-					 && iCodeInputCount) {
-				    iCodeInputCount--;
-				    strCodeInput[iCodeInputCount] = '\0';
-				    retVal = IRV_DISPLAY_MESSAGE;
-				    if (!iCodeInputCount)
-					retVal = IRV_CLEAN;
-				}
-				uMessageUp = 1;
-				uMessageDown = 1;
-				if (bEngAfterSemicolon && !iCodeInputCount) {
-				    strcpy (messageUp[0].strMsg, "Ӣ״̬");
-				    strcpy (messageDown[0].strMsg, "ո룻Enter;");
-				}
-				else {
-				    strcpy (messageUp[0].strMsg, strCodeInput);
-				    strcpy (messageDown[0].strMsg, " Enter/ո Ӣ");
-				}
-				messageUp[0].type = MSG_INPUT;
-				messageDown[0].type = MSG_TIPS;
-			    }
-			    else if ((bLastIsNumber && bEngPuncAfterNumber)
-				     && (iKey == '.' || iKey == ',')
-				     && !iCandWordCount) {
+			if (iKey == CTRL_LSHIFT || iKey == SHIFT_LCTRL) {
+			    if (bLocked)
 				retVal = IRV_TO_PROCESS;
-				bLastIsNumber = False;
+			}
+			else {
+			    //뷨ģ
+			    if (!iInCap) {
+				retVal = im[iIMIndex].DoInput (iKey);
+				if (!IsIM (NAME_OF_PINYIN) && !IsIM (NAME_OF_SHUANGPIN))
+				    iCursorPos = iCodeInputCount;
+			    }
+			    else if (iInCap == 2 && semicolonToDo == K_SEMICOLON_QUICKPHRASE) {
+				retVal = QuickPhraseDoInput (iKey);
 			    }
-			    else {
-				if (bChnPunc) {
-				    int             iPunc;
 
-				    pstr = NULL;
-				    iPunc = IsPunc (iKey);
-				    if (iPunc != -1) {
-					strStringGet[0] = '\0';
-					if (!bIsInLegend)
-					    pstr = im[iIMIndex].GetCandWord (0);
-					if (pstr)
+			    if (!bIsDoInputOnly && retVal == IRV_TO_PROCESS) {
+				if (iKey >= 'A' && iKey <= 'Z' && bEngAfterCap && !(kev->state & KEY_CAPSLOCK)) {
+				    iInCap = 1;
+				    if (!bIsInLegend && iCandWordCount) {
+					pstr = im[iIMIndex].GetCandWord (0);
+					iCandWordCount = 0;
+					if (pstr) {
 					    strcpy (strStringGet, pstr);
-					strcat (strStringGet, chnPunc[iPunc].strChnPunc[chnPunc[iPunc].iWhich]);
-
-					uMessageUp = 1;
-					messageUp[0].strMsg[0] = iKey;
-					messageUp[0].strMsg[1] = '\0';
-					messageUp[0].type = MSG_INPUT;
-
-					uMessageDown = 1;
-					strcpy (messageDown[0].strMsg, chnPunc[iPunc].strChnPunc[chnPunc[iPunc].iWhich]);
-					messageDown[0].type = MSG_OTHER;
-
-					chnPunc[iPunc].iWhich++;
-					if (chnPunc[iPunc].iWhich >= chnPunc[iPunc].iCount)
-					    chnPunc[iPunc].iWhich = 0;
-
-					retVal = IRV_PUNC;
+					    //ͳ
+					    // iHZInputed += (int) (strlen (strStringGet) / 2);
+					    SendHZtoClient (call_data, strStringGet);
+					    iCodeInputCount = 0;
+					}
 				    }
-				    else if (isprint (iKey) && iKey < 128) {
-					if (iKey >= '0' && iKey <= '9')
-					    bLastIsNumber = True;
+				}
+				else if (iKey == ';' && semicolonToDo != K_SEMICOLON_NOCHANGE && !iCodeInputCount)
+				    iInCap = 2;
+				else if (IsHotKey (iKey, hkPrevPage)) {
+				    if (iInCap == 2 && semicolonToDo == K_SEMICOLON_QUICKPHRASE)
+					retVal = QuickPhraseGetCandWords (SM_PREV);
+				    else
+					retVal = im[iIMIndex].GetCandWords (SM_PREV);
+				}
+				else if (IsHotKey (iKey, hkNextPage)) {
+				    if (iInCap == 2 && semicolonToDo == K_SEMICOLON_QUICKPHRASE)
+					retVal = QuickPhraseGetCandWords (SM_NEXT);
+				    else
+					retVal = im[iIMIndex].GetCandWords (SM_NEXT);
+				}
+
+				if (retVal == IRV_TO_PROCESS) {
+				    if (iInCap) {
+					if (iKey == ' ') {
+					    if (iCodeInputCount == 0)
+						strcpy (strStringGet, "");
+					    else
+						strcpy (strStringGet, strCodeInput);
+					    retVal = IRV_ENG;
+					    uMessageUp = uMessageDown = 0;
+					    iInCap = 0;
+					}
 					else {
-					    bLastIsNumber = False;
-					    if (iKey == ' ')
-						retVal = IRV_DONOT_PROCESS_CLEAN;	//Ϊmozilla
+					    if (isprint (iKey) && iKey < 128) {
+						if (iCodeInputCount == MAX_USER_INPUT)
+						    retVal = IRV_DO_NOTHING;
+						else {
+						    if (!(iInCap == 2 && !iCodeInputCount && iKey == ';')) {
+							strCodeInput[iCodeInputCount++] = iKey;
+							strCodeInput[iCodeInputCount] = '\0';
+							bShowCursor = True;
+							iCursorPos = iCodeInputCount;
+							if (semicolonToDo == K_SEMICOLON_QUICKPHRASE && iInCap == 2)
+							    retVal = QuickPhraseGetCandWords (SM_FIRST);
+							else
+							    retVal = IRV_DISPLAY_MESSAGE;
+						    }
+						    else
+							retVal = IRV_DISPLAY_MESSAGE;
+						}
+					    }
+					    else if ((iKey == (XK_BackSpace & 0x00FF) || iKey == CTRL_H) && iCodeInputCount) {
+						iCodeInputCount--;
+						strCodeInput[iCodeInputCount] = '\0';
+						iCursorPos = iCodeInputCount;
+						retVal = IRV_DISPLAY_MESSAGE;
+						if (!iCodeInputCount)
+						    retVal = IRV_CLEAN;
+						else if (semicolonToDo == K_SEMICOLON_QUICKPHRASE && iInCap == 2)
+						    retVal = QuickPhraseGetCandWords (SM_FIRST);
+					    }
+
+					    uMessageUp = 1;
+					    if (iInCap == 2) {
+						strcpy (messageUp[0].strMsg, strCodeInput);
+						if (semicolonToDo == K_SEMICOLON_ENG)
+						    strcat (messageUp[0].strMsg, "Ӣ룺");
+						else {
+						    if (iCodeInputCount)
+							strcat (messageUp[0].strMsg, "  ");
+						    strcat (messageUp[0].strMsg, "Զ");
+						}
+						if (!iCodeInputCount) {
+						    strcpy (messageDown[0].strMsg, "ո룻Enter;");
+						    uMessageDown = 1;
+						    messageDown[0].type = MSG_TIPS;
+						}
+					    }
 					    else {
+						uMessageDown = 1;
+						messageDown[0].type = MSG_TIPS;
+						strcpy (messageUp[0].strMsg, strCodeInput);
+						strcpy (messageDown[0].strMsg, " Enter/ո Ӣ");
+					    }
+					    messageUp[0].type = MSG_INPUT;
+					}
+				    }
+				    else if ((bLastIsNumber && bEngPuncAfterNumber) && (iKey == '.' || iKey == ',') && !iCandWordCount) {
+					retVal = IRV_TO_PROCESS;
+					bLastIsNumber = False;
+				    }
+				    else {
+					if (bChnPunc) {
+					    char           *pPunc;
+
+					    pstr = NULL;
+					    pPunc = GetPunc (iKey);
+					    if (pPunc) {
 						strStringGet[0] = '\0';
 						if (!bIsInLegend)
 						    pstr = im[iIMIndex].GetCandWord (0);
 						if (pstr)
 						    strcpy (strStringGet, pstr);
-						iLen = strlen (strStringGet);
+						strcat (strStringGet, pPunc);
 						uMessageDown = uMessageUp = 0;
-						strStringGet[iLen] = iKey;
-						strStringGet[iLen + 1] = '\0';
-						retVal = IRV_ENG;
+
+						retVal = IRV_PUNC;
+					    }
+					    else if (isprint (iKey) && iKey < 128) {
+						if (iKey >= '0' && iKey <= '9')
+						    bLastIsNumber = True;
+						else {
+						    bLastIsNumber = False;
+						    if (iKey == ' ')
+							retVal = IRV_DONOT_PROCESS_CLEAN;	//Ϊmozilla
+						    else {
+							strStringGet[0] = '\0';
+							if (!bIsInLegend)
+							    pstr = im[iIMIndex].GetCandWord (0);
+							if (pstr)
+							    strcpy (strStringGet, pstr);
+							iLen = strlen (strStringGet);
+							uMessageDown = uMessageUp = 0;
+							strStringGet[iLen] = iKey;
+							strStringGet[iLen + 1] = '\0';
+							retVal = IRV_ENG;
+						    }
+						}
 					    }
 					}
 				    }
 				}
 			    }
-			}
-		    }
 
-		    if (retVal == IRV_TO_PROCESS) {
-			if (iKey == ESC) {
-			    if (iCodeInputCount || bInCap || bIsInLegend)
-				retVal = IRV_CLEAN;
-			    else
-				retVal = IRV_DONOT_PROCESS;
-			}
-			else if (iKey == CTRL_5) {
-			    SetIM ();
-			    LoadConfig (False);
-
-			    if (bLumaQQ)
-				ConnectIDResetReset ();
-
-			    retVal = IRV_DO_NOTHING;
-			}
-			else if (iKey == ENTER) {
-			    if (bInCap) {
-				if (bEngAfterSemicolon && !iCodeInputCount)
-				    strcpy (strStringGet, ";");
-				else
-				    strcpy (strStringGet, strCodeInput);
-				retVal = IRV_ENG;
-				uMessageUp = uMessageDown = 0;
-				bInCap = False;
-			    }
-			    else if (!iCodeInputCount)
-				retVal = IRV_DONOT_PROCESS;
-			    else {
-				switch (enterToDo) {
-				case K_ENTER_NOTHING:
-				    retVal = IRV_DO_NOTHING;
-				    break;
-				case K_ENTER_CLEAN:
-				    retVal = IRV_CLEAN;
-				    break;
-				case K_ENTER_SEND:
-				    uMessageDown = 1;
-				    strcpy (messageDown[0].strMsg, strCodeInput);
-				    strcpy (strStringGet, strCodeInput);
-				    retVal = IRV_ENG;
-				    break;
+			    if (retVal == IRV_TO_PROCESS) {
+				if (iKey == ESC) {
+				    if (iCodeInputCount || iInCap || bIsInLegend)
+					retVal = IRV_CLEAN;
+				    else
+					retVal = IRV_DONOT_PROCESS;
 				}
+				else if (iKey == CTRL_5) {
+				    LoadConfig (False);
+				    SetIM ();
+
+				    /*if (bLumaQQ)
+				       ConnectIDResetReset (); */
+
+				    CreateFont ();
+				    SwitchIM (iIMIndex);
+				    CalculateInputWindowHeight ();
+
+				    FreeQuickPhrase ();
+				    LoadQuickPhrase ();
+
+				    FreePunc ();
+				    LoadPuncDict ();
+
+				    retVal = IRV_DO_NOTHING;
+				}
+				else if (iKey == ENTER) {
+				    if (iInCap) {
+					if (!iCodeInputCount)
+					    strcpy (strStringGet, ";");
+					else
+					    strcpy (strStringGet, strCodeInput);
+					retVal = IRV_PUNC;
+					uMessageUp = uMessageDown = 0;
+					iInCap = 0;
+				    }
+				    else if (!iCodeInputCount)
+					retVal = IRV_DONOT_PROCESS;
+				    else {
+					switch (enterToDo) {
+					case K_ENTER_NOTHING:
+					    retVal = IRV_DO_NOTHING;
+					    break;
+					case K_ENTER_CLEAN:
+					    retVal = IRV_CLEAN;
+					    break;
+					case K_ENTER_SEND:
+					    uMessageDown = uMessageUp = 0;
+					    strcpy (strStringGet, strCodeInput);
+					    retVal = IRV_ENG;
+					    break;
+					}
+				    }
+				}
+				else if (isprint (iKey) && iKey < 128)
+				    retVal = IRV_DONOT_PROCESS_CLEAN;
+				else
+				    retVal = IRV_DONOT_PROCESS;
 			    }
 			}
-			else if (isprint (iKey) && iKey < 128)
-			    retVal = IRV_DONOT_PROCESS_CLEAN;
-			else
-			    retVal = IRV_DONOT_PROCESS;
 		    }
 		}
+		else {
+		    if (bCorner && (iKey >= 32 && iKey <= 126)) {
+			//˱ ո ȫǲԣȷ0xa1 0xa1
+			//ȴ˵ȫǷ0xa3ʼ
+			//0xa3 0xa0ܻʾ룬˲0xa1 0xa1ķʽ
+			if (iKey == ' ')
+			    sprintf (strStringGet, "%c%c", 0xa1, 0xa1);
+			else
+			    sprintf (strStringGet, "%c%c", 0xa3, 0xa0 + iKey - 32);
+			retVal = IRV_GET_CANDWORDS;
+		    }
+		}
+
+		if (retVal == IRV_TO_PROCESS || retVal == IRV_DONOT_PROCESS) {
+		    if (IsHotKey (iKey, hkCorner))
+			retVal = ChangeCorner ();
+		    else if (IsHotKey (iKey, hkPunc))
+			retVal = ChangePunc ();
+		    else if (IsHotKey (iKey, hkGBK))
+			retVal = ChangeGBK ();
+		    else if (IsHotKey (iKey, hkLegend))
+			retVal = ChangeLegend ();
+		    else if (IsHotKey (iKey, hkTrack))
+			retVal = ChangeTrack ();
+		}
 	    }
-	    else if (call_data->event.type != KeyPress)
+	    else
 		retVal = IRV_DONOT_PROCESS;
 	}
     }
@@ -561,7 +679,8 @@ void ProcessKey (IMForwardEventStruct * call_data)
     case IRV_DONOT_PROCESS:
     case IRV_DONOT_PROCESS_CLEAN:
 	if (call_data->event.type == KeyRelease) {
-	    if (!bLumaQQ && (!keyCount || (!iKeyState && (iKey == ESC || iKey == ENTER))))
+	    // if (!bLumaQQ && (!keyCount || (!iKeyState && (iKey == ESC || iKey == ENTER))))
+	    if (!keyCount || (!iKeyState && (iKey == ESC || iKey == ENTER)))
 		IMForwardEvent (ims, (XPointer) call_data);
 	}
 	else
@@ -613,6 +732,7 @@ void ProcessKey (IMForwardEventStruct * call_data)
 	break;
     case IRV_GET_LEGEND:
 	SendHZtoClient (call_data, strStringGet);
+	// iHZInputed += (int) (strlen (strStringGet) / 2);     //ͳ
 	if (iLegendCandWordCount) {
 	    bShowNext = bShowPrev = False;
 	    if (iCurrentLegendCandPage > 0)
@@ -633,19 +753,26 @@ void ProcessKey (IMForwardEventStruct * call_data)
 
 	break;
     case IRV_GET_CANDWORDS:
-	if (bPhraseTips && im[iIMIndex].PhraseTips)
+	if (bPhraseTips && im[iIMIndex].PhraseTips && !bVK)
 	    DoPhraseTips ();
     case IRV_ENG:
+	//ıģʽӦýеıתΪȫ
+	if (retVal != IRV_GET_CANDWORDS && bChnPunc && bConvertPunc)
+	    ConvertPunc ();
     case IRV_PUNC:
+	//iHZInputed += (int) (strlen (strStringGet) / 2);      //ͳ
 	ResetInput ();
-	if (!(uMessageDown && retVal == IRV_GET_CANDWORDS)
-	    && bAutoHideInputWindow && (retVal == IRV_PUNC || (!bPhraseTips || (bPhraseTips && !lastIsSingleHZ))))
+	if (bVK || (!(uMessageDown && retVal == IRV_GET_CANDWORDS)
+		    && bAutoHideInputWindow && (retVal == IRV_PUNC || (!bPhraseTips || (bPhraseTips && !lastIsSingleHZ)))))
 	    XUnmapWindow (dpy, inputWindow);
-	else
+	else if (ConnectIDGetState (call_data->connect_id) == IS_CHN)
 	    DisplayInputWindow ();
     case IRV_GET_CANDWORDS_NEXT:
-	if (retVal == IRV_GET_CANDWORDS_NEXT || lastIsSingleHZ == -1)
+	if (retVal == IRV_GET_CANDWORDS_NEXT || lastIsSingleHZ == -1) {
+	    // iHZInputed += (int) (strlen (strStringGet) / 2);  //ͳ
 	    DisplayInputWindow ();
+	}
+
 	SendHZtoClient (call_data, strStringGet);
 	bLastIsNumber = False;
 	lastIsSingleHZ = 0;
@@ -654,6 +781,15 @@ void ProcessKey (IMForwardEventStruct * call_data)
     default:
 	;
     }
+
+    /* ٶȵĹ
+       if (retVal == IRV_DISPLAY_MESSAGE || retVal == IRV_DISPLAY_CANDWORDS || retVal == IRV_PUNC) {
+       if (!bStartRecordType) {
+       bStartRecordType = True;
+       timeStart = time (NULL);
+       }
+       }
+     */
 }
 
 Bool IsHotKey (int iKey, HOTKEYS * hotkey)
@@ -699,6 +835,20 @@ INPUT_RETURN_VALUE ChangeGBK (void)
     return IRV_CLEAN;
 }
 
+INPUT_RETURN_VALUE ChangeGBKT (void)
+{
+    bUseGBKT = !bUseGBKT;
+    ResetInput ();
+    ResetInputWindow ();
+
+    DisplayMainWindow ();
+    XUnmapWindow (dpy, inputWindow);
+
+    SaveProfile ();
+
+    return IRV_CLEAN;
+}
+
 INPUT_RETURN_VALUE ChangeLegend (void)
 {
     bUseLegend = !bUseLegend;
@@ -725,28 +875,36 @@ void SwitchIM (INT8 index)
 {
     INT8            iLastIM;
 
-    iLastIM = (iIMIndex >= iIMCount) ? (iIMCount - 1) : iIMIndex;
+    if (index != (INT8) - 2 && bVK)
+	return;
 
+    iLastIM = (iIMIndex >= iIMCount) ? (iIMCount - 1) : iIMIndex;
     if (index == (INT8) - 1) {
 	if (iIMIndex == (iIMCount - 1))
 	    iIMIndex = 0;
 	else
 	    iIMIndex++;
     }
-    else {
+    else if (index != (INT8) - 2) {
 	if (index >= iIMCount)
 	    iIMIndex = iIMCount - 1;
     }
 
 #ifdef _USE_XFT
-    MAINWND_WIDTH = ((bCompactMainWindow) ? _MAINWND_WIDTH_COMPACT : _MAINWND_WIDTH) + StringWidth (im[iIMIndex].strName, xftMainWindowFont) + 4;
+    MAINWND_WIDTH = ((bCompactMainWindow) ? _MAINWND_WIDTH_COMPACT : _MAINWND_WIDTH) + StringWidth ((bVK) ? vks[iCurrentVK].strName : im[iIMIndex].strName, xftMainWindowFont) + 4;
 #else
-    MAINWND_WIDTH = ((bCompactMainWindow) ? _MAINWND_WIDTH_COMPACT : _MAINWND_WIDTH) + StringWidth (im[iIMIndex].strName, fontSetMainWindow) + 4;
+    MAINWND_WIDTH = ((bCompactMainWindow) ? _MAINWND_WIDTH_COMPACT : _MAINWND_WIDTH) + StringWidth ((bVK) ? vks[iCurrentVK].strName : im[iIMIndex].strName, fontSetMainWindow) + 4;
 #endif
+    if (!bShowVK && bCompactMainWindow)
+	MAINWND_WIDTH -= 24;
+
     XResizeWindow (dpy, mainWindow, MAINWND_WIDTH, MAINWND_HEIGHT);
 
     DisplayMainWindow ();
 
+    if (im[iIMIndex].Init)
+	im[iIMIndex].Init ();
+
     if (iIMCount == 1)
 	return;
 
@@ -757,9 +915,6 @@ void SwitchIM (INT8 index)
     XUnmapWindow (dpy, inputWindow);
 
     SaveProfile ();
-
-    if (im[iIMIndex].Init)
-	im[iIMIndex].Init ();
 }
 
 void DoPhraseTips (void)
