@@ -1,3 +1,22 @@
+/***************************************************************************
+ *   Copyright (C) 2002~2005 by Yuking                                     *
+ *   yuking_net@sohu.com                                                   *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc.,                                       *
+ *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ ***************************************************************************/
 #include <stdio.h>
 #include <sys/stat.h>
 #include <limits.h>
@@ -85,6 +104,8 @@ SP_S            SPMap_S[] = {
 Bool            bSP_UseSemicolon = False;
 Bool            bSP = False;
 char            cNonS = 'o';
+char	    	strDefaultSP[100]="Ȼ";
+SP_FROM	        iSPFrom=SP_FROM_USER;	//Ӻδȡ˫ƴ0ûĿ¼1ϵͳã2ϵͳ˫ƴļ
 
 extern Bool     bSingleHZMode;
 
@@ -100,23 +121,39 @@ void LoadSPData (void)
 {
     FILE           *fp;
     char            strPath[PATH_MAX];
-    char            str[20], strS[5], *pstr;
+    char            str[100], strS[5], *pstr;
     int             i;
+    Bool	    bIsDefault=False;
+    Bool	    bIsFromSystemSPConfig=False;
 
     strcpy (strPath, (char *) getenv ("HOME"));
     strcat (strPath, "/.fcitx/");
 
     if (access (strPath, 0))
 	mkdir (strPath, S_IRWXU);
-
+    
+    strcpy (strPath, (char *) getenv ("HOME"));
+    strcat (strPath, "/.fcitx/");
     strcat (strPath, "sp.dat");
+
+    if (access (strPath, 0)) {
+	strcpy (strPath, PKGDATADIR "/data/");
+	strcat (strPath, "sp.dat");
+	bIsFromSystemSPConfig = True;
+	if (iSPFrom != SP_FROM_SYSTEM_CONFIG )
+	    iSPFrom = SP_FROM_SYSTEM_SP_CONFIG;
+    }
+
     fp = fopen (strPath, "rt");
 
     if (!fp)
 	return;
 
+    if ( !bIsFromSystemSPConfig )
+	iSPFrom = SP_FROM_USER;
+    
     while (1) {
-	if (!fgets (str, 20, fp))
+	if (!fgets (str, 100, fp))
 	    break;
 
 	i = strlen (str) - 1;
@@ -127,7 +164,28 @@ void LoadSPData (void)
 	    pstr++;
 	if (!strlen (pstr) || pstr[0] == '#')
 	    continue;
-
+	
+	if (!strncmp(pstr,"ĬϷ=",9)) {
+		if (iSPFrom!=SP_FROM_SYSTEM_CONFIG) {
+		    pstr+=9;
+		    if (*pstr == ' ' || *pstr == '\t')
+			pstr++;
+		    strcpy(strDefaultSP,pstr);
+		}
+		continue;
+	}
+	
+	if (!strncmp(pstr,"=",9)) {
+		pstr+=9;
+		if (*pstr == ' ' || *pstr == '\t')
+			pstr++;
+		bIsDefault = !(strcmp(strDefaultSP,pstr));
+		continue;
+	}
+	
+	if ( !bIsDefault )
+		continue;
+	
 	if (pstr[0] == '=')	//ĸ
 	    cNonS = tolower (pstr[1]);
 	else {
