@@ -1,3 +1,22 @@
+/***************************************************************************
+ *   Copyright (C) 2002~2005 by Yuking                                     *
+ *   yuking_net@sohu.com                                                   *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc.,                                       *
+ *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ ***************************************************************************/
 #ifndef _TABLE_H
 #define _TABLE_H
 
@@ -11,7 +30,7 @@
 #define MAX_CODE_LENGTH  12
 #define PHRASE_MAX_LENGTH 10
 #define FH_MAX_LENGTH  10
-#define TABLE_AUTO_SAVE_AFTER 48
+#define TABLE_AUTO_SAVE_AFTER 1024
 
 typedef struct _RULE_RULE {
     unsigned char   iFlag;	// 1 -->    0 --> 
@@ -42,16 +61,15 @@ typedef struct _TABLE {
     unsigned int    iRecordCount;
     ADJUSTORDER     tableOrder;
 
-    Bool            bUsePY;	//ʹƴ
-    Bool	    bGetPY;	//ƴ
+    Bool            bUsePY;			//ʹƴ
     Bool            bTableAutoSendToClient;	//Զ
-    Bool            bUseMatchingKey;	//Ƿģƥ
-    Bool            bAutoPhrase;	//ǷԶ
+    Bool            bUseMatchingKey;		//Ƿģƥ
+    Bool            bAutoPhrase;		//ǷԶ
     INT8            iSaveAutoPhraseAfter;	//ѡNκ󱣴Զ飬0-棬1-
-    Bool            bAutoPhrasePhrase;	//Ƿ
-    INT8            iAutoPhrase;	//Զʳ
-    Bool            bTableExactMatch;	//Ƿֻʾȷƥĺѡ/
-    Bool            bPromptTableCode;	//ϺǷʾ--ʹƴʱʾ
+    Bool            bAutoPhrasePhrase;		//Ƿ
+    INT8            iAutoPhrase;		//Զʳ
+    Bool            bTableExactMatch;		//Ƿֻʾȷƥĺѡ/
+    Bool            bPromptTableCode;		//ϺǷʾ
 } TABLE;
 
 typedef struct _RECORD {
