@@ -1,3 +1,22 @@
+/***************************************************************************
+ *   Copyright (C) 2002~2005 by Yuking                                     *
+ *   yuking_net@sohu.com                                                   *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc.,                                       *
+ *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ ***************************************************************************/
 #include "tools.h"
 
 #include <stdio.h>
@@ -28,6 +47,7 @@ extern int      iTempInputWindowY;
 extern int      iInputWindowWidth;
 extern int      iInputWindowHeight;
 
+extern char     strUserLocale[];
 extern int      iMaxCandWord;
 extern Bool     _3DEffectMainWindow;
 extern _3D_EFFECT _3DEffectInputWindow;
@@ -38,6 +58,9 @@ extern MESSAGE_COLOR messageColor[];
 extern MESSAGE_COLOR inputWindowLineColor;
 extern MESSAGE_COLOR mainWindowLineColor;
 extern MESSAGE_COLOR cursorColor;
+extern WINDOW_COLOR VKWindowColor;
+extern MESSAGE_COLOR VKWindowFontColor;
+extern MESSAGE_COLOR VKWindowAlphaColor;
 extern ENTER_TO_DO enterToDo;
 
 extern HOTKEYS  hkTrigger[];
@@ -53,7 +76,8 @@ extern HOTKEYS  hkPYAddFreq[];
 extern HOTKEYS  hkPYDelFreq[];
 extern HOTKEYS  hkPYDelUserPhr[];
 extern HOTKEYS  hkLegend[];
-extern HOTKEYS	hkTrack[];
+extern HOTKEYS  hkTrack[];
+extern HOTKEYS  hkGetPY[];
 
 extern KEYCODE  switchKey;
 extern XIMTriggerKey *Trigger_Keys;
@@ -63,13 +87,16 @@ extern Bool     bUseGBK;
 extern Bool     bEngPuncAfterNumber;
 extern Bool     bAutoHideInputWindow;
 extern XColor   colorArrow;
-extern Bool	bTrackCursor;
+extern Bool     bTrackCursor;
+extern Bool     bCenterInputWindow;
 extern HIDE_MAINWINDOW hideMainWindow;
 extern Bool     bCompactMainWindow;
 extern HIDE_MAINWINDOW hideMainWindow;
 extern int      iFontSize;
 extern int      iMainWindowFontSize;
 
+extern Bool     bUseGBKT;
+
 extern Bool     bChnPunc;
 extern Bool     bCorner;
 extern Bool     bUseLegend;
@@ -77,9 +104,14 @@ extern Bool     bUseLegend;
 extern Bool     bPYCreateAuto;
 extern Bool     bPYSaveAutoAsPhrase;
 extern Bool     bPhraseTips;
-extern Bool     bEngAfterSemicolon;
+extern SEMICOLON_TO_DO semicolonToDo;
 extern Bool     bEngAfterCap;
 
+/*
+extern Bool     bShowUserSpeed;
+*/
+extern Bool     bShowVK;
+
 extern Bool     bFullPY;
 extern Bool     bDisablePagingInLegend;
 
@@ -104,10 +136,18 @@ extern Bool     bUseSP;
 extern Bool     bUseQW;
 extern Bool     bUseTable;
 
-extern Bool	bLumaQQ;
-extern char	cPYYCDZ[];
+extern char	strDefaultSP[];
+extern SP_FROM	iSPFrom;
 
-extern Bool	bDoubleSwitchKey;
+//extern Bool     bLumaQQ;
+extern char     cPYYCDZ[];
+
+extern Bool     bDoubleSwitchKey;
+extern Bool     bPointAfterNumber;
+extern Bool     bConvertPunc;
+extern unsigned int iTimeInterval;
+extern uint     iFixedInputWindowWidth;
+extern Bool     bShowInputWindowTriggering;
 
 #ifdef _USE_XFT
 extern Bool     bUseAA;
@@ -115,6 +155,8 @@ extern Bool     bUseAA;
 extern char     strUserLocale[];
 #endif
 
+extern INT8     iOffsetY;
+
 /*
 #if defined(DARWIN)*/
 /* function to reverse byte order for integer
@@ -131,6 +173,12 @@ this is required for Mac machine*/
 }
 #endif
 */
+
+Bool MyStrcmp (char *str1, char *str2)
+{
+    return !strncmp (str1, str2, strlen (str2));
+}
+
 /*
  * ȡûļ
  */
@@ -141,14 +189,22 @@ void LoadConfig (Bool bMode)
     char            strPath[PATH_MAX];
     int             i;
     int             r, g, b;	//
+    Bool	    bFromUser=True;
 
     strcpy (strPath, (char *) getenv ("HOME"));
     strcat (strPath, "/.fcitx/config");
 
+    if (access (strPath, 0)) {
+	strcpy (strPath, PKGDATADIR "/data/config");
+	bFromUser = False;
+    }
+
     fp = fopen (strPath, "rt");
 
+    if (!fp || !bFromUser)
+	SaveConfig();
+
     if (!fp) {
-	SaveConfig ();
 	LoadConfig (True);	//Ĭֵ
 	return;
     }
@@ -167,85 +223,121 @@ void LoadConfig (Bool bMode)
 	if (pstr[0] == '#')
 	    continue;
 
-	if (strstr (pstr, "ʾ()=") && bMode) {
+	if (MyStrcmp (pstr, "=")) {
+	    pstr += 9;
+	    if (*pstr == ' ')
+		pstr++;
+	    strcpy (strUserLocale, pstr);
+	}
+	else if (MyStrcmp (pstr, "ʾ()=")) {
 	    pstr += 13;
 	    strcpy (strFontName, pstr);
 	}
-	if (strstr (pstr, "ʾ(Ӣ)=") && bMode) {
+	else if (MyStrcmp (pstr, "ʾ(Ӣ)=")) {
 	    pstr += 13;
 	    strcpy (strFontEnName, pstr);
 	}
-	else if (strstr (pstr, "ʾС=") && bMode) {
+	else if (MyStrcmp (pstr, "ʾС=")) {
 	    pstr += 13;
 	    iFontSize = atoi (pstr);
 	}
-	else if (strstr (pstr, "С=") && bMode) {
+	else if (MyStrcmp (pstr, "С=")) {
 	    pstr += 15;
 	    iMainWindowFontSize = atoi (pstr);
 	}
 #ifdef _USE_XFT
-	else if (strstr (pstr, "ǷʹAA=") && bMode) {
+	else if (MyStrcmp (pstr, "ǷʹAA=")) {
 	    pstr += 15;
 	    bUseAA = atoi (pstr);
 	}
 #else
-	else if (strstr (pstr, "=") && bMode) {
+	else if (MyStrcmp (pstr, "=")) {
 	    pstr += 9;
 	    strcpy (strUserLocale, pstr);
 	}
 #endif
-	else if (strstr (pstr, "ѡʸ=")) {
+	else if (MyStrcmp (pstr, "ѡʸ=")) {
 	    pstr += 11;
 	    iMaxCandWord = atoi (pstr);
 	    if (iMaxCandWord > 10)
 		iMaxCandWord = MAX_CAND_WORD;
 	}
-	else if (strstr (pstr, "ֺǷ=")) {
+	else if (MyStrcmp (pstr, "ֺǷ=")) {
 	    pstr += 17;
 	    bEngPuncAfterNumber = atoi (pstr);
 	}
-	else if (strstr (pstr, "LumaQQ֧=") ) {
-	    pstr += 11;
-	    bLumaQQ = atoi(pstr);
-	}
-	else if (strstr (pstr, "Ƿʹ3D=")) {
+	/*else if (MyStrcmp (pstr, "LumaQQ֧=")) {
+	   pstr += 11;
+	   bLumaQQ = atoi (pstr);
+	   } */
+	else if (MyStrcmp (pstr, "Ƿʹ3D=")) {
 	    pstr += 21;
 	    _3DEffectMainWindow = atoi (pstr);
 	}
-	else if (strstr (pstr, "ʹ3D=")) {
+	else if (MyStrcmp (pstr, "ʹ3D=")) {
 	    pstr += 17;
-	    _3DEffectInputWindow = (_3D_EFFECT)atoi (pstr);
+	    _3DEffectInputWindow = (_3D_EFFECT) atoi (pstr);
 	}
-	else if (strstr (pstr, "ǷԶ=")) {
+	else if (MyStrcmp (pstr, "ǷԶ=")) {
 	    pstr += 19;
 	    bAutoHideInputWindow = atoi (pstr);
 	}
-	else if (strstr (pstr, "ģʽ=")) {
+	else if (MyStrcmp (pstr, "Ƿ=")) {
+	    pstr += 15;
+	    bCenterInputWindow = atoi (pstr);
+	}
+	else if (MyStrcmp (pstr, "̶=")) {
+	    pstr += 15;
+	    iFixedInputWindowWidth = atoi (pstr);
+	}
+	else if (MyStrcmp (pstr, "״ʾ=")) {
+	    pstr += 15;
+	    bShowInputWindowTriggering = atoi (pstr);
+	}
+	// ʱ
+	else if (MyStrcmp (pstr, "߶ƫ=")) {
+	    pstr += 17;
+	    iOffsetY = atoi (pstr);
+	}
+	//********************************************
+	else if (MyStrcmp (pstr, "źӵ=")) {
+	    pstr += 11;
+	    bPointAfterNumber = atoi (pstr);
+	}
+	/*else if (MyStrcmp (pstr, "ʾٶ=")) {
+	   pstr += 13;
+	   bShowUserSpeed = atoi (pstr);
+	   } */
+	else if (MyStrcmp (pstr, "ģʽ=")) {
 	    pstr += 15;
 	    hideMainWindow = (HIDE_MAINWINDOW) atoi (pstr);
 	}
-	else if (strstr (pstr, "ɫ=") && bMode) {
+	else if (MyStrcmp (pstr, "ʾ=")) {
+	    pstr += 13;
+	    bShowVK = atoi (pstr);
+	}
+	else if (MyStrcmp (pstr, "ɫ=") && bMode) {
 	    pstr += 7;
 	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
 	    cursorColor.color.red = (r << 8);
 	    cursorColor.color.green = (g << 8);
 	    cursorColor.color.blue = (b << 8);
 	}
-	else if (strstr (pstr, "ڱɫ=") && bMode) {
+	else if (MyStrcmp (pstr, "ڱɫ=") && bMode) {
 	    pstr += 13;
 	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
 	    mainWindowColor.backColor.red = (r << 8);
 	    mainWindowColor.backColor.green = (g << 8);
 	    mainWindowColor.backColor.blue = (b << 8);
 	}
-	else if (strstr (pstr, "ɫ=") && bMode) {
+	else if (MyStrcmp (pstr, "ɫ=") && bMode) {
 	    pstr += 13;
 	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
 	    mainWindowLineColor.color.red = (r << 8);
 	    mainWindowLineColor.color.green = (g << 8);
 	    mainWindowLineColor.color.blue = (b << 8);
 	}
-	else if (strstr (pstr, "뷨ɫ=") && bMode) {
+	else if (MyStrcmp (pstr, "뷨ɫ=") && bMode) {
 	    int             r1, r2, g1, g2, b1, b2;
 
 	    pstr += 19;
@@ -260,131 +352,168 @@ void LoadConfig (Bool bMode)
 	    IMNameColor[2].color.green = (g2 << 8);
 	    IMNameColor[2].color.blue = (b2 << 8);
 	}
-	else if (strstr (pstr, "봰ɫ=") && bMode) {
+	else if (MyStrcmp (pstr, "봰ɫ=") && bMode) {
 	    pstr += 13;
 	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
 	    inputWindowColor.backColor.red = (r << 8);
 	    inputWindowColor.backColor.green = (g << 8);
 	    inputWindowColor.backColor.blue = (b << 8);
 	}
-	else if (strstr (pstr, "봰ʾɫ=") && bMode) {
+	else if (MyStrcmp (pstr, "봰ʾɫ=") && bMode) {
 	    pstr += 13;
 	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
 	    messageColor[0].color.red = (r << 8);
 	    messageColor[0].color.green = (g << 8);
 	    messageColor[0].color.blue = (b << 8);
 	}
-	else if (strstr (pstr, "봰ûɫ=") && bMode) {
+	else if (MyStrcmp (pstr, "봰ûɫ=") && bMode) {
 	    pstr += 17;
 	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
 	    messageColor[1].color.red = (r << 8);
 	    messageColor[1].color.green = (g << 8);
 	    messageColor[1].color.blue = (b << 8);
 	}
-	else if (strstr (pstr, "봰ɫ=") && bMode) {
+	else if (MyStrcmp (pstr, "봰ɫ=") && bMode) {
 	    pstr += 13;
 	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
 	    messageColor[2].color.red = (r << 8);
 	    messageColor[2].color.green = (g << 8);
 	    messageColor[2].color.blue = (b << 8);
 	}
-	else if (strstr (pstr, "봰һѡɫ=") && bMode) {
+	else if (MyStrcmp (pstr, "봰һѡɫ=") && bMode) {
 	    pstr += 21;
 	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
 	    messageColor[3].color.red = (r << 8);
 	    messageColor[3].color.green = (g << 8);
 	    messageColor[3].color.blue = (b << 8);
 	}
-	else if (strstr (pstr, "봰ûɫ=") && bMode) {
+	else if (MyStrcmp (pstr, "봰ûɫ=") && bMode) {
 	    pstr += 17;
 	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
 	    messageColor[4].color.red = (r << 8);
 	    messageColor[4].color.green = (g << 8);
 	    messageColor[4].color.blue = (b << 8);
 	}
-	else if (strstr (pstr, "봰ʾɫ=") && bMode) {
+	else if (MyStrcmp (pstr, "봰ʾɫ=") && bMode) {
 	    pstr += 17;
 	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
 	    messageColor[5].color.red = (r << 8);
 	    messageColor[5].color.green = (g << 8);
 	    messageColor[5].color.blue = (b << 8);
 	}
-	else if (strstr (pstr, "봰ıɫ=") && bMode) {
+	else if (MyStrcmp (pstr, "봰ıɫ=") && bMode) {
 	    pstr += 17;
 	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
 	    messageColor[6].color.red = (r << 8);
 	    messageColor[6].color.green = (g << 8);
 	    messageColor[6].color.blue = (b << 8);
 	}
-	else if (strstr (pstr, "봰ɫ=") && bMode) {
+	else if (MyStrcmp (pstr, "봰ɫ=") && bMode) {
 	    pstr += 13;
 	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
 	    inputWindowLineColor.color.red = (r << 8);
 	    inputWindowLineColor.color.green = (g << 8);
 	    inputWindowLineColor.color.blue = (b << 8);
 	}
-	else if (strstr (pstr, "봰ͷɫ=") && bMode) {
+	else if (MyStrcmp (pstr, "봰ͷɫ=") && bMode) {
 	    pstr += 13;
 	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
 	    colorArrow.red = (r << 8);
 	    colorArrow.green = (g << 8);
 	    colorArrow.blue = (b << 8);
 	}
+	else if (MyStrcmp (pstr, "̴ɫ=") && bMode) {
+	    pstr += 17;
+	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
+	    VKWindowColor.backColor.red = (r << 8);
+	    VKWindowColor.backColor.green = (g << 8);
+	    VKWindowColor.backColor.blue = (b << 8);
+	}
+	else if (MyStrcmp (pstr, "̴ĸɫ=") && bMode) {
+	    pstr += 17;
+	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
+	    VKWindowAlphaColor.color.red = (r << 8);
+	    VKWindowAlphaColor.color.green = (g << 8);
+	    VKWindowAlphaColor.color.blue = (b << 8);
+	}
+	else if (MyStrcmp (pstr, "̴ɫ=") && bMode) {
+	    pstr += 17;
+	    sscanf (pstr, "%d %d %d\n", &r, &g, &b);
+	    VKWindowFontColor.color.red = (r << 8);
+	    VKWindowFontColor.color.green = (g << 8);
+	    VKWindowFontColor.color.blue = (b << 8);
+	}
 
-	else if (strstr (pstr, "/ر뷨=") && bMode) {
+	else if (MyStrcmp (pstr, "/ر뷨=") && bMode) {
 	    pstr += 16;
 	    SetTriggerKeys (pstr);
 	    SetHotKey (pstr, hkTrigger);
 	}
-	else if (strstr (pstr, "ӢĿл=")) {
+	else if (MyStrcmp (pstr, "ӢĿл=")) {
 	    pstr += 17;
 	    SetSwitchKey (pstr);
 	}
-	else if (strstr (pstr, "˫Ӣл=")) {
+	else if (MyStrcmp (pstr, "˫Ӣл=")) {
 	    pstr += 15;
 	    bDoubleSwitchKey = atoi (pstr);
 	}
-	else if (strstr (pstr, "GBK֧=")) {
+	else if (MyStrcmp (pstr, "ʱ=")) {
+	    pstr += 13;
+	    iTimeInterval = atoi (pstr);
+	}
+	else if (MyStrcmp (pstr, "=")) {
+	    pstr += 9;
+	    SetHotKey (pstr, hkTrack);
+	}
+	else if (MyStrcmp (pstr, "GBK֧=")) {
 	    pstr += 8;
 	    SetHotKey (pstr, hkGBK);
 	}
-	else if (strstr (pstr, "ֺӢ=")) {
-	    pstr += 13;
-	    bEngAfterSemicolon = atoi (pstr);
+	else if (MyStrcmp (pstr, "ֺżΪ=")) {
+	    pstr += 11;
+	    semicolonToDo = atoi (pstr);
 	}
-	else if (strstr (pstr, "дĸӢ=")) {
+	else if (MyStrcmp (pstr, "дĸӢ=")) {
 	    pstr += 17;
 	    bEngAfterCap = atoi (pstr);
 	}
-	else if (strstr (pstr, "뷽ʽֹҳ=")) {
+	else if (MyStrcmp (pstr, "תӢеı=")) {
+	    pstr += 17;
+	    bConvertPunc = atoi (pstr);
+	}
+	else if (MyStrcmp (pstr, "뷽ʽֹҳ=")) {
 	    pstr += 17;
 	    bDisablePagingInLegend = atoi (pstr);
 	}
-	else if (strstr (pstr, "֧=")) {
+	else if (MyStrcmp (pstr, "֧=")) {
 	    pstr += 9;
 	    SetHotKey (pstr, hkLegend);
 	}
-	else if (strstr (pstr, "EnterΪ=")) {
+	else if (MyStrcmp (pstr, "ƴ=")) {
+	    pstr += 9;
+	    SetHotKey (pstr, hkGetPY);
+	}
+	else if (MyStrcmp (pstr, "EnterΪ=")) {
 	    pstr += 12;
 	    enterToDo = (ENTER_TO_DO) atoi (pstr);
 	}
-	else if (strstr (pstr, "ȫ=")) {
+	else if (MyStrcmp (pstr, "ȫ=")) {
 	    pstr += 7;
 	    SetHotKey (pstr, hkCorner);
 	}
-	else if (strstr (pstr, "ı=")) {
+	else if (MyStrcmp (pstr, "ı=")) {
 	    pstr += 9;
 	    SetHotKey (pstr, hkPunc);
 	}
-	else if (strstr (pstr, "һҳ=")) {
+	else if (MyStrcmp (pstr, "һҳ=")) {
 	    pstr += 7;
 	    SetHotKey (pstr, hkPrevPage);
 	}
-	else if (strstr (pstr, "һҳ=")) {
+	else if (MyStrcmp (pstr, "һҳ=")) {
 	    pstr += 7;
 	    SetHotKey (pstr, hkNextPage);
 	}
-	else if (strstr (pstr, "ڶѡѡ=")) {
+	else if (MyStrcmp (pstr, "ڶѡѡ=")) {
 	    pstr += 19;
 	    if (!strcasecmp (pstr, "SHIFT")) {
 		i2ndSelectKey = 50;	//SHIFTɨ
@@ -396,110 +525,117 @@ void LoadConfig (Bool bMode)
 	    }
 	}
 
-	else if (strstr (pstr, "ʹƴ=")) {
+	else if (MyStrcmp (pstr, "ʹƴ=")) {
 	    pstr += 9;
 	    bUsePinyin = atoi (pstr);
 	}
-	else if (strstr (pstr, "ʹ˫ƴ=")) {
+	else if (MyStrcmp (pstr, "ʹ˫ƴ=")) {
 	    pstr += 9;
 	    bUseSP = atoi (pstr);
 	}
-	else if (strstr (pstr, "ʹλ=")) {
+	else if (MyStrcmp (pstr, "Ĭ˫ƴ=")) {
+	    pstr += 13;
+	    if (*pstr == ' ')
+		pstr++;
+	    strcpy (strDefaultSP, pstr);
+	    iSPFrom = SP_FROM_SYSTEM_CONFIG;
+	}
+	else if (MyStrcmp (pstr, "ʹλ=")) {
 	    pstr += 9;
 	    bUseQW = atoi (pstr);
 	}
-	else if (strstr (pstr, "ʹ=")) {
+	else if (MyStrcmp (pstr, "ʹ=")) {
 	    pstr += 9;
 	    bUseTable = atoi (pstr);
 	}
-	else if (strstr (pstr, "ʾʿеĴ=")) {
+	else if (MyStrcmp (pstr, "ʾʿеĴ=")) {
 	    pstr += 17;
 	    bPhraseTips = atoi (pstr);
 	}
 
-	else if (strstr (str, "ʹȫƴ=")) {
+	else if (MyStrcmp (str, "ʹȫƴ=")) {
 	    pstr += 9;
 	    bFullPY = atoi (pstr);
 	}
-	else if (strstr (str, "ƴԶ=")) {
+	else if (MyStrcmp (str, "ƴԶ=")) {
 	    pstr += 13;
 	    bPYCreateAuto = atoi (pstr);
 	}
-	else if (strstr (str, "Զ=")) {
+	else if (MyStrcmp (str, "Զ=")) {
 	    pstr += 13;
 	    bPYSaveAutoAsPhrase = atoi (pstr);
 	}
-	else if (strstr (str, "ƴ=")) {
+	else if (MyStrcmp (str, "ƴ=")) {
 	    pstr += 15;
 	    SetHotKey (pstr, hkPYAddFreq);
 	}
-	else if (strstr (str, "ɾƴ=")) {
+	else if (MyStrcmp (str, "ɾƴ=")) {
 	    pstr += 15;
 	    SetHotKey (pstr, hkPYDelFreq);
 	}
-	else if (strstr (str, "ɾƴû=")) {
+	else if (MyStrcmp (str, "ɾƴû=")) {
 	    pstr += 17;
 	    SetHotKey (pstr, hkPYDelUserPhr);
 	}
-	else if (strstr (str, "ƴԴʶּ=")) {
+	else if (MyStrcmp (str, "ƴԴʶּ=")) {
 	    pstr += 15;
 	    cPYYCDZ[0] = pstr[0];
 	    cPYYCDZ[1] = pstr[1];
 	}
-	else if (strstr (str, "ƴʽ=")) {
+	else if (MyStrcmp (str, "ƴʽ=")) {
 	    pstr += 21;
 	    baseOrder = (ADJUSTORDER) atoi (pstr);
 	}
-	else if (strstr (str, "ƴʽ=")) {
+	else if (MyStrcmp (str, "ƴʽ=")) {
 	    pstr += 21;
 	    phraseOrder = (ADJUSTORDER) atoi (pstr);
 	}
-	else if (strstr (str, "ƴôʽ=")) {
+	else if (MyStrcmp (str, "ƴôʽ=")) {
 	    pstr += 23;
 	    freqOrder = (ADJUSTORDER) atoi (pstr);
 	}
-	else if (strstr (str, "Ƿģanang=")) {
+	else if (MyStrcmp (str, "Ƿģanang=")) {
 	    pstr += 16;
 	    MHPY_C[0].bMode = atoi (pstr);
 	    MHPY_S[5].bMode = atoi (pstr);
 	}
-	else if (strstr (str, "Ƿģeneng=")) {
+	else if (MyStrcmp (str, "Ƿģeneng=")) {
 	    pstr += 16;
 	    MHPY_C[1].bMode = atoi (pstr);
 	}
-	else if (strstr (str, "Ƿģianiang=")) {
+	else if (MyStrcmp (str, "Ƿģianiang=")) {
 	    pstr += 18;
 	    MHPY_C[2].bMode = atoi (pstr);
 	}
-	else if (strstr (str, "Ƿģining=")) {
+	else if (MyStrcmp (str, "Ƿģining=")) {
 	    pstr += 16;
 	    MHPY_C[3].bMode = atoi (pstr);
 	}
-	else if (strstr (str, "Ƿģouu=")) {
+	else if (MyStrcmp (str, "Ƿģouu=")) {
 	    pstr += 14;
 	    MHPY_C[4].bMode = atoi (pstr);
 	}
-	else if (strstr (str, "Ƿģuanuang=")) {
+	else if (MyStrcmp (str, "Ƿģuanuang=")) {
 	    pstr += 18;
 	    MHPY_C[5].bMode = atoi (pstr);
 	}
-	else if (strstr (str, "Ƿģcch=")) {
+	else if (MyStrcmp (str, "Ƿģcch=")) {
 	    pstr += 14;
 	    MHPY_S[0].bMode = atoi (pstr);
 	}
-	else if (strstr (str, "Ƿģfh=")) {
+	else if (MyStrcmp (str, "Ƿģfh=")) {
 	    pstr += 13;
 	    MHPY_S[1].bMode = atoi (pstr);
 	}
-	else if (strstr (str, "Ƿģln=")) {
+	else if (MyStrcmp (str, "Ƿģln=")) {
 	    pstr += 13;
 	    MHPY_S[2].bMode = atoi (pstr);
 	}
-	else if (strstr (str, "Ƿģssh=")) {
+	else if (MyStrcmp (str, "Ƿģssh=")) {
 	    pstr += 14;
 	    MHPY_S[3].bMode = atoi (pstr);
 	}
-	else if (strstr (str, "Ƿģzzh=")) {
+	else if (MyStrcmp (str, "Ƿģzzh=")) {
 	    pstr += 14;
 	    MHPY_S[4].bMode = atoi (pstr);
 	}
@@ -542,30 +678,37 @@ void SaveConfig (void)
     fprintf (fp, "ʾ(Ӣ)=%s\n", strFontEnName);
     fprintf (fp, "ʾС=%d\n", iFontSize);
     fprintf (fp, "С=%d\n", iMainWindowFontSize);
+    if (strUserLocale[0])
+	fprintf (fp, "=%s\n", strUserLocale);
+    else
+	fprintf (fp, "#=zh_CN.gb18030\n");
 #ifdef _USE_XFT
     fprintf (fp, "ǷʹAA=%d\n", bUseAA);
-#else
-    if ( strUserLocale[0] )
-    	fprintf (fp, "=%s\n", strUserLocale);
-    else
-    	fprintf (fp, "#=zh_CN.gb2312\n");
 #endif
 
     fprintf (fp, "\n[]\n");
     fprintf (fp, "ֺǷ=%d\n", bEngPuncAfterNumber);
     fprintf (fp, "EnterΪ=%d\n", enterToDo);
-    fprintf (fp, "ֺӢ=%d\n", bEngAfterSemicolon);
+    fprintf (fp, "ֺżΪ=%d\n", (int) semicolonToDo);
     fprintf (fp, "дĸӢ=%d\n", bEngAfterCap);
+    fprintf (fp, "תӢеı=%d\n", bConvertPunc);
     fprintf (fp, "뷽ʽֹҳ=%d\n", bDisablePagingInLegend);
-    fprintf (fp, "LumaQQ֧=%d\n", bLumaQQ);
+    //fprintf (fp, "LumaQQ֧=%d\n", bLumaQQ);
 
     fprintf (fp, "\n[]\n");
     fprintf (fp, "ѡʸ=%d\n", iMaxCandWord);
     fprintf (fp, "Ƿʹ3D=%d\n", _3DEffectMainWindow);
     fprintf (fp, "ʹ3D=%d\n", _3DEffectInputWindow);
     fprintf (fp, "ģʽ=%d\n", (int) hideMainWindow);
+    fprintf (fp, "ʾ=%d\n", (int) bShowVK);
     fprintf (fp, "ǷԶ=%d\n", bAutoHideInputWindow);
-    
+    fprintf (fp, "Ƿ=%d\n", bCenterInputWindow);
+    fprintf (fp, "״ʾ=%d\n", bShowInputWindowTriggering);
+    fprintf (fp, "#̶(뷨)0ʾ̶\n");
+    fprintf (fp, "̶=%d\n", iFixedInputWindowWidth);
+    fprintf (fp, "źӵ=%d\n", bPointAfterNumber);
+    //fprintf (fp, "ʾٶ=%d\n", bShowUserSpeed);
+
     fprintf (fp, "ɫ=%d %d %d\n", cursorColor.color.red >> 8, cursorColor.color.green >> 8, cursorColor.color.blue >> 8);
     fprintf (fp, "ڱɫ=%d %d %d\n", mainWindowColor.backColor.red >> 8, mainWindowColor.backColor.green >> 8, mainWindowColor.backColor.blue >> 8);
     fprintf (fp, "ɫ=%d %d %d\n", mainWindowLineColor.color.red >> 8, mainWindowLineColor.color.green >> 8, mainWindowLineColor.color.blue >> 8);
@@ -583,6 +726,9 @@ void SaveConfig (void)
     fprintf (fp, "봰ıɫ=%d %d %d\n", messageColor[6].color.red >> 8, messageColor[6].color.green >> 8, messageColor[6].color.blue >> 8);
     fprintf (fp, "봰ɫ=%d %d %d\n", inputWindowLineColor.color.red >> 8, inputWindowLineColor.color.green >> 8, inputWindowLineColor.color.blue >> 8);
     fprintf (fp, "봰ͷɫ=%d %d %d\n", colorArrow.red >> 8, colorArrow.green >> 8, colorArrow.blue >> 8);
+    fprintf (fp, "̴ɫ=%d %d %d\n", VKWindowColor.backColor.red >> 8, VKWindowColor.backColor.green >> 8, VKWindowColor.backColor.blue >> 8);
+    fprintf (fp, "̴ĸɫ=%d %d %d\n", VKWindowAlphaColor.color.red >> 8, VKWindowAlphaColor.color.green >> 8, VKWindowAlphaColor.color.blue >> 8);
+    fprintf (fp, "̴ɫ=%d %d %d\n", VKWindowFontColor.color.red >> 8, VKWindowFontColor.color.green >> 8, VKWindowFontColor.color.blue >> 8);
 
     fprintf (fp, "\n#ˡӢĿл⣬ȼΪмÿոָ\n");
     fprintf (fp, "[ȼ]\n");
@@ -590,18 +736,21 @@ void SaveConfig (void)
     fprintf (fp, "#ӢĿл ΪL_CTRL R_CTRL L_SHIFT R_SHIFT\n");
     fprintf (fp, "ӢĿл=L_CTRL\n");
     fprintf (fp, "˫Ӣл=%d\n", bDoubleSwitchKey);
+    fprintf (fp, "ʱ=%u\n", iTimeInterval);
     fprintf (fp, "=CTRL_K\n");
     fprintf (fp, "GBK֧=CTRL_M\n");
     fprintf (fp, "֧=CTRL_L\n");
+    fprintf (fp, "ƴ=CTRL_ALT_E\n");
     fprintf (fp, "ȫ=SHIFT_SPACE\n");
     fprintf (fp, "ı=ALT_SPACE\n");
-    fprintf (fp, "һҳ=-\n");
-    fprintf (fp, "һҳ==\n");
+    fprintf (fp, "һҳ=- ,\n");
+    fprintf (fp, "һҳ== .\n");
     fprintf (fp, "ڶѡѡ=SHIFT\n");
 
     fprintf (fp, "\n[뷨]\n");
     fprintf (fp, "ʹƴ=%d\n", bUsePinyin);
     fprintf (fp, "ʹ˫ƴ=%d\n", bUseSP);
+    fprintf (fp, "Ĭ˫ƴ=%s\n",strDefaultSP);
     fprintf (fp, "ʹλ=%d\n", bUseQW);
     fprintf (fp, "ʹ=%d\n", bUseTable);
     fprintf (fp, "ʾʿеĴ=%d\n", bPhraseTips);
@@ -664,13 +813,13 @@ void LoadProfile (void)
 
 	    pstr = str;
 
-	    if (strstr (str, "汾=")) {
+	    if (MyStrcmp (str, "汾=")) {
 		pstr += 5;
 
 		if (!strcasecmp (FCITX_VERSION, pstr))
 		    bRetVal = True;
 	    }
-	    else if (strstr (str, "λX=")) {
+	    else if (MyStrcmp (str, "λX=")) {
 		pstr += 12;
 		iMainWindowX = atoi (pstr);
 		if (iMainWindowX < 0)
@@ -678,7 +827,7 @@ void LoadProfile (void)
 		else if ((iMainWindowX + MAINWND_WIDTH) > DisplayWidth (dpy, iScreen))
 		    iMainWindowX = DisplayWidth (dpy, iScreen) - MAINWND_WIDTH;
 	    }
-	    else if (strstr (str, "λY=")) {
+	    else if (MyStrcmp (str, "λY=")) {
 		pstr += 12;
 		iMainWindowY = atoi (pstr);
 		if (iMainWindowY < 0)
@@ -686,7 +835,7 @@ void LoadProfile (void)
 		else if ((iMainWindowY + MAINWND_HEIGHT) > DisplayHeight (dpy, iScreen))
 		    iMainWindowY = DisplayHeight (dpy, iScreen) - MAINWND_HEIGHT;
 	    }
-	    else if (strstr (str, "봰λX=")) {
+	    else if (MyStrcmp (str, "봰λX=")) {
 		pstr += 14;
 		iInputWindowX = atoi (pstr);
 		if (iInputWindowX < 0)
@@ -694,7 +843,7 @@ void LoadProfile (void)
 		else if ((iInputWindowX + iInputWindowWidth) > DisplayWidth (dpy, iScreen))
 		    iInputWindowX = DisplayWidth (dpy, iScreen) - iInputWindowWidth - 3;
 	    }
-	    else if (strstr (str, "봰λY=")) {
+	    else if (MyStrcmp (str, "봰λY=")) {
 		pstr += 14;
 		iInputWindowY = atoi (pstr);
 		if (iInputWindowY < 0)
@@ -702,38 +851,42 @@ void LoadProfile (void)
 		else if ((iInputWindowY + INPUTWND_HEIGHT) > DisplayHeight (dpy, iScreen))
 		    iInputWindowY = DisplayHeight (dpy, iScreen) - iInputWindowHeight;
 	    }
-	    else if (strstr (str, "Ƿȫ=")) {
+	    else if (MyStrcmp (str, "Ƿȫ=")) {
 		pstr += 9;
 		bCorner = atoi (pstr);
 	    }
-	    else if (strstr (str, "Ƿı=")) {
+	    else if (MyStrcmp (str, "Ƿı=")) {
 		pstr += 13;
 		bChnPunc = atoi (pstr);
 	    }
-	    else if (strstr (str, "ǷGBK=")) {
+	    else if (MyStrcmp (str, "ǷGBK=")) {
 		pstr += 8;
 		bUseGBK = atoi (pstr);
 	    }
-	    else if (strstr (str, "Ƿ=")) {
+	    else if (MyStrcmp (str, "Ƿ=")) {
 		pstr += 13;
 		bTrackCursor = atoi (pstr);
 	    }
-	    else if (strstr (str, "Ƿ=")) {
+	    else if (MyStrcmp (str, "Ƿ=")) {
 		pstr += 9;
 		bUseLegend = atoi (pstr);
 	    }
-	    else if (strstr (str, "ǰ뷨=")) {
+	    else if (MyStrcmp (str, "ǰ뷨=")) {
 		pstr += 11;
 		iIMIndex = atoi (pstr);
 	    }
-	    else if (strstr (str, "ֹüл=")) {
+	    else if (MyStrcmp (str, "ֹüл=")) {
 		pstr += 15;
 		bLocked = atoi (pstr);
 	    }
-	    else if (strstr (str, "ڼģʽ=")) {
-		pstr += 15;
+	    else if (MyStrcmp (str, "ģʽ=")) {
+		pstr += 9;
 		bCompactMainWindow = atoi (pstr);
 	    }
+	    else if (MyStrcmp (str, "ǷGBK=")) {
+		pstr += 16;
+		bUseGBKT = atoi (pstr);
+	    }
 	}
 
 	fclose (fp);
@@ -776,8 +929,10 @@ void SaveProfile (void)
     fprintf (fp, "Ƿ=%d\n", bUseLegend);
     fprintf (fp, "ǰ뷨=%d\n", iIMIndex);
     fprintf (fp, "ֹüл=%d\n", bLocked);
-    fprintf (fp, "ڼģʽ=%d\n", bCompactMainWindow);
-    
+    fprintf (fp, "ģʽ=%d\n", bCompactMainWindow);
+
+    fprintf (fp, "ǷGBK=%d\n", bUseGBKT);
+
     iTempInputWindowX = iInputWindowX;
     iTempInputWindowY = iInputWindowY;
 
@@ -889,15 +1044,15 @@ void SetTriggerKeys (char *str)
     }
 
     for (i = 0; i <= iTriggerKeyCount; i++) {
-	if (strstr (strKey[i], "CTRL_")) {
+	if (MyStrcmp (strKey[i], "CTRL_")) {
 	    Trigger_Keys[i].modifier = Trigger_Keys[i].modifier | ControlMask;
 	    Trigger_Keys[i].modifier_mask = Trigger_Keys[i].modifier_mask | ControlMask;
 	}
-	else if (strstr (strKey[i], "SHIFT_")) {
+	else if (MyStrcmp (strKey[i], "SHIFT_")) {
 	    Trigger_Keys[i].modifier = Trigger_Keys[i].modifier | ShiftMask;
 	    Trigger_Keys[i].modifier_mask = Trigger_Keys[i].modifier_mask | ShiftMask;
 	}
-	else if (strstr (strKey[i], "ALT_")) {
+	else if (MyStrcmp (strKey[i], "ALT_")) {
 	    Trigger_Keys[i].modifier = Trigger_Keys[i].modifier | Mod1Mask;
 	    Trigger_Keys[i].modifier_mask = Trigger_Keys[i].modifier_mask | Mod1Mask;
 	}
@@ -953,3 +1108,75 @@ Bool CheckHZCharset (char *strHZ)
 
     return True;
 }
+
+static char    *gGBKS2TTable = NULL;
+static int      gGBKS2TTableSize = -1;
+
+/**
+ * úװdata/gbks2t.tabļת
+ * ȻGBKַתGBKַ
+ *
+ * WARNING ú·ڴַ
+ * עͷš
+ */
+char           *ConvertGBKSimple2Tradition (char *strHZ)
+{
+    FILE           *fp;
+    char           *ret;
+    char            strPath[PATH_MAX];
+    int             i, len;
+    unsigned int    idx;
+
+    if (strHZ == NULL)
+	return NULL;
+
+    if (!gGBKS2TTable) {
+	len = 0;
+
+	strcpy (strPath, PKGDATADIR "/data/");
+	strcat (strPath, TABLE_GBKS2T);
+	fp = fopen (strPath, "rb");
+	if (!fp) {
+	    ret = (char *) malloc (sizeof (char) * (strlen (strHZ) + 1));
+	    strcpy (ret, strHZ);
+	    return ret;
+	}
+
+	fseek (fp, 0, SEEK_END);
+	fgetpos (fp, (fpos_t *) & len);
+	if (len > 0) {
+	    gGBKS2TTable = (char *) malloc (sizeof (char) * len);
+	    gGBKS2TTableSize = len;
+	    fseek (fp, 0, SEEK_SET);
+	    fread (gGBKS2TTable, sizeof (char), len, fp);
+	}
+	fclose (fp);
+    }
+
+    i = 0;
+    len = strlen (strHZ);
+    ret = (char *) malloc (sizeof (char) * (len + 1));
+    for (; i < len; ++i) {
+	if (i < (len - 1))
+	    if ((unsigned char) strHZ[i] >= (unsigned char) 0x81
+		&& (unsigned char) strHZ[i] <= (unsigned char) 0xfe &&
+		(((unsigned char) strHZ[i + 1] >= (unsigned char) 0x40 && (unsigned char) strHZ[i + 1] <= (unsigned char) 0x7e) || ((unsigned char) strHZ[i + 1] > (unsigned char) 0x7f && (unsigned char) strHZ[i + 1] <= (unsigned char) 0xfe))) {
+		idx = (((unsigned char) strHZ[i] - (unsigned char) 0x81)
+		       * (unsigned char) 0xbe + ((unsigned char) strHZ[i + 1] - (unsigned char) 0x40)
+		       - ((unsigned char) strHZ[i + 1] / (unsigned char) 0x80)) * 2;
+		if (idx >= 0 && idx < gGBKS2TTableSize - 1) {
+		    if (	/*(unsigned char)gGBKS2TTable[idx] != (unsigned char)0xa1 
+				   && */ (unsigned char) gGBKS2TTable[idx + 1] != (unsigned char) 0x7f) {
+			ret[i] = gGBKS2TTable[idx];
+			ret[i + 1] = gGBKS2TTable[idx + 1];
+			i += 1;
+			continue;
+		    }
+		}
+	    }
+	ret[i] = strHZ[i];
+    }
+    ret[len] = '\0';
+
+    return ret;
+}
