@@ -1,3 +1,22 @@
+/***************************************************************************
+ *   Copyright (C) 2002~2005 by Yuking                                     *
+ *   yuking_net@sohu.com                                                   *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc.,                                       *
+ *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ ***************************************************************************/
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -10,23 +29,26 @@
 #include "tools.h"
 #include "MainWindow.h"
 #include "InputWindow.h"
+#include "vk.h"
 
 CONNECT_ID     *connectIDsHead = (CONNECT_ID *) NULL;
-XIMS		ims;
+XIMS            ims;
+
 //************************************************
 CARD16          connect_id = 0;
+CARD16          icid = 0;
 CARD16          lastConnectID = 0;
 
 //************************************************
 
 long            filter_mask = KeyPressMask | KeyReleaseMask;
 IC             *CurrentIC = NULL;
-Bool            bBackground = True;
 char            strLocale[201] = "zh_CN.GB18030,zh_CN.GB2312,zh_CN,zh_CN.GBK,zh_CN.UTF-8,zh_CN.UTF8,en_US.UTF-8,en_US.UTF8";
 
-//int y=0;
+//ñGTK+ OverTheSpotʱ
+INT8            iOffsetY = 12;
 
-extern Bool     bLumaQQ;
+//extern Bool     bLumaQQ;
 
 extern IM      *im;
 extern INT8     iIMIndex;
@@ -35,6 +57,7 @@ extern Display *dpy;
 extern int      iScreen;
 extern Window   mainWindow;
 extern Window   inputWindow;
+extern Window   VKWindow;
 
 extern int      iMainWindowX;
 extern int      iMainWindowY;
@@ -45,10 +68,20 @@ extern int      iTempInputWindowY;
 extern uint     iInputWindowHeight;
 extern uint     iInputWindowWidth;
 extern Bool     bTrackCursor;
+extern Bool     bCenterInputWindow;
 extern Bool     bIsUtf8;
 extern int      iCodeInputCount;
 extern iconv_t  convUTF8;
 extern uint     uMessageDown;
+extern Bool     bVK;
+
+/* ٶ
+extern Bool     bStartRecordType;
+extern uint     iHZInputed;
+*/
+extern Bool     bShowInputWindowTriggering;
+
+extern Bool     bUseGBKT;
 
 /*extern char	strUserLocale[];*/
 //+++++++++++++++++++++++++++++++++
@@ -93,7 +126,7 @@ static XIMEncoding zhEncodings[] = {
 Bool MyOpenHandler (IMOpenStruct * call_data)
 {
     CreateConnectID (call_data);
-    
+
     return True;
 }
 
@@ -129,13 +162,13 @@ Bool MySetICValuesHandler (IMChangeICStruct * call_data)
 		    iTempInputWindowX = 0;
 		else if ((iTempInputWindowX + iInputWindowWidth) > DisplayWidth (dpy, iScreen))
 		    iTempInputWindowX = DisplayWidth (dpy, iScreen) - iInputWindowWidth;
-		else
-		    iTempInputWindowX += 3;
 
 		if (iTempInputWindowY < 0)
 		    iTempInputWindowY = 0;
 		else if ((iTempInputWindowY + iInputWindowHeight) > DisplayHeight (dpy, iScreen))
 		    iTempInputWindowY = DisplayHeight (dpy, iScreen) - iInputWindowHeight;
+		else
+		    iTempInputWindowY += iOffsetY;
 
 		XMoveWindow (dpy, inputWindow, iTempInputWindowX, iTempInputWindowY);
 
@@ -143,6 +176,12 @@ Bool MySetICValuesHandler (IMChangeICStruct * call_data)
 	    }
 	}
     }
+    else if (bCenterInputWindow) {
+	iInputWindowX = (DisplayWidth (dpy, iScreen) - iInputWindowWidth) / 2;
+	if (iInputWindowX < 0)
+	    iInputWindowX = 0;
+	XMoveWindow (dpy, inputWindow, iInputWindowX, iInputWindowY);
+    }
 
     SetIC (call_data);
 
@@ -153,28 +192,79 @@ Bool MySetFocusHandler (IMChangeFocusStruct * call_data)
 {
     CurrentIC = (IC *) FindIC (call_data->icid);
     connect_id = call_data->connect_id;
+    icid = call_data->icid;
 
-    if (ConnectIDGetState (call_data->connect_id) != IS_CLOSED) {
+    /* It seems this section is useless
+       if (bTrackCursor) {
+       int             i;
+       Window          window;
+       XICAttribute   *pre_attr = ((IMChangeICStruct *) call_data)->preedit_attr;
+
+       for (i = 0; i < (int) ((IMChangeICStruct *) call_data)->preedit_attr_num; i++, pre_attr++) {
+       if (!strcmp (XNSpotLocation, pre_attr->name)) {
+       if (CurrentIC->focus_win)
+       XTranslateCoordinates (dpy, CurrentIC->client_win, RootWindow (dpy, iScreen), (*(XPoint *) pre_attr->value).x, (*(XPoint *) pre_attr->value).y, &iTempInputWindowX, &iTempInputWindowY, &window);
+       else if (CurrentIC->client_win)
+       XTranslateCoordinates (dpy, CurrentIC->client_win, RootWindow (dpy, iScreen), (*(XPoint *) pre_attr->value).x, (*(XPoint *) pre_attr->value).y, &iTempInputWindowX, &iTempInputWindowY, &window);
+       else
+       return True;
+
+       if (iTempInputWindowX < 0)
+       iTempInputWindowX = 0;
+       else if ((iTempInputWindowX + iInputWindowWidth) > DisplayWidth (dpy, iScreen))
+       iTempInputWindowX = DisplayWidth (dpy, iScreen) - iInputWindowWidth;
+       else
+       iTempInputWindowX += 5;
+
+       if (iTempInputWindowY < 0)
+       iTempInputWindowY = 0;
+       else if ((iTempInputWindowY + iInputWindowHeight) > DisplayHeight (dpy, iScreen))
+       iTempInputWindowY = DisplayHeight (dpy, iScreen) - iInputWindowHeight;
+
+       XMoveWindow (dpy, inputWindow, iTempInputWindowX, iTempInputWindowY);
+
+       ConnectIDSetTrackCursor (call_data->connect_id, True);
+       }
+       }
+       } */
+    /* ************************************************************************ */
+
+    if (ConnectIDGetState (connect_id) != IS_CLOSED) {
 	IMPreeditStart (ims, (XPointer) call_data);
 	EnterChineseMode (lastConnectID == connect_id);
-	if (ConnectIDGetState (call_data->connect_id) == IS_CHN && uMessageDown > 1 && lastConnectID == connect_id)
-	    DisplayInputWindow ();
-	else
+	if (ConnectIDGetState (connect_id) == IS_CHN) {
+	    if (bVK)
+		DisplayVKWindow ();
+	    else if (uMessageDown > 1 && lastConnectID == connect_id)
+		DisplayInputWindow ();
+	}
+	else {
 	    XUnmapWindow (dpy, inputWindow);
+	    XUnmapWindow (dpy, VKWindow);
+	}
 
 	if (ConnectIDGetTrackCursor (connect_id))
 	    XMoveWindow (dpy, inputWindow, iTempInputWindowX, iTempInputWindowY);
 	else
 	    XMoveWindow (dpy, inputWindow, iInputWindowX, iInputWindowY);
     }
-    else
+    else {
+	XUnmapWindow (dpy, inputWindow);
+	XUnmapWindow (dpy, VKWindow);
 	DisplayMainWindow ();
+    }
 
     lastConnectID = connect_id;
-    if (bLumaQQ && ConnectIDGetReset (connect_id)) {
-	SendHZtoClient ((IMForwardEventStruct *) call_data, "\0");
-	ConnectIDSetReset (connect_id, False);
-    }
+    /*if (bLumaQQ && ConnectIDGetReset (connect_id)) {
+       SendHZtoClient ((IMForwardEventStruct *) call_data, "\0");
+       ConnectIDSetReset (connect_id, False);
+       } */
+
+    //When application gets the focus, rerecord the time.
+    /*
+       bStartRecordType = False;
+       iHZInputed = 0;
+     */
 
     return True;
 }
@@ -182,6 +272,7 @@ Bool MySetFocusHandler (IMChangeFocusStruct * call_data)
 Bool MyUnsetFocusHandler (IMChangeICStruct * call_data)
 {
     XUnmapWindow (dpy, inputWindow);
+    XUnmapWindow (dpy, VKWindow);
 
     return True;
 }
@@ -189,8 +280,10 @@ Bool MyUnsetFocusHandler (IMChangeICStruct * call_data)
 Bool MyCloseHandler (IMOpenStruct * call_data)
 {
     XUnmapWindow (dpy, inputWindow);
+    XUnmapWindow (dpy, VKWindow);
     DestroyConnectID (call_data->connect_id);
     connect_id = 0;
+    icid = 0;
 
     return True;
 }
@@ -202,6 +295,7 @@ Bool MyCreateICHandler (IMChangeICStruct * call_data)
     if (!CurrentIC) {
 	CurrentIC = (IC *) FindIC (call_data->icid);;
 	connect_id = call_data->connect_id;
+	icid = call_data->icid;
     }
 
     return True;
@@ -209,8 +303,10 @@ Bool MyCreateICHandler (IMChangeICStruct * call_data)
 
 Bool MyDestroyICHandler (IMChangeICStruct * call_data)
 {
-    if (CurrentIC == (IC *) FindIC (call_data->icid))
+    if (CurrentIC == (IC *) FindIC (call_data->icid)) {
 	XUnmapWindow (dpy, inputWindow);
+	XUnmapWindow (dpy, VKWindow);
+    }
 
     DestroyIC (call_data);
 
@@ -245,6 +341,9 @@ Bool MyTriggerNotifyHandler (IMTriggerNotifyStruct * call_data)
 	    XMoveWindow (dpy, inputWindow, iTempInputWindowX, iTempInputWindowY);
 	else
 	    XMoveWindow (dpy, inputWindow, iInputWindowX, iInputWindowY);
+
+	if (bShowInputWindowTriggering)
+	    DisplayInputWindow ();
     }
 
     return True;
@@ -323,9 +422,12 @@ Bool MyProtoHandler (XIMS _ims, IMProtocol * call_data)
 void SendHZtoClient (IMForwardEventStruct * call_data, char *strHZ)
 {
     XTextProperty   tp;
-    Display        *display = ims->core.display;
     char            strOutput[300];
     char           *ps;
+    char           *pS2T = (char *) NULL;
+
+    if (bUseGBKT)
+	pS2T = strHZ = ConvertGBKSimple2Tradition (strHZ);
 
     if (bIsUtf8) {
 	size_t          l1, l2;
@@ -333,18 +435,20 @@ void SendHZtoClient (IMForwardEventStruct * call_data, char *strHZ)
 	ps = strOutput;
 	l1 = strlen (strHZ);
 	l2 = 299;
-	l1 = iconv (convUTF8, (char **) (&strHZ), &l1, &ps, &l2);
+	l1 = iconv (convUTF8, (ICONV_CONST char **) (&strHZ), &l1, &ps, &l2);
 	*ps = '\0';
 	ps = strOutput;
     }
     else
 	ps = strHZ;
 
-    XmbTextListToTextProperty (display, (char **) &ps, 1, XCompoundTextStyle, &tp);
+    XmbTextListToTextProperty (dpy, (char **) &ps, 1, XCompoundTextStyle, &tp);
     ((IMCommitStruct *) call_data)->flag |= XimLookupChars;
     ((IMCommitStruct *) call_data)->commit_string = (char *) tp.value;
     IMCommitString (ims, (XPointer) call_data);
     XFree (tp.value);
+    if (bUseGBKT)
+	free (pS2T);
 }
 
 Bool InitXIM (char *imname, Window im_window)
@@ -410,26 +514,31 @@ Bool InitXIM (char *imname, Window im_window)
 
 void SetIMState (Bool bState)
 {
-	IMChangeFocusStruct call_data;
-	
-	if ( !CurrentIC )
-		return;
-	
-	if ( connect_id && CurrentIC->id ) {
-		call_data.connect_id = connect_id;
-		call_data.icid = CurrentIC->id;
-	
-		if ( bState ) {
-			IMPreeditStart (ims, (XPointer) &call_data);
-			SetConnectID (connect_id, IS_CHN);
-		}
-		else {
-			IMPreeditEnd (ims, (XPointer) &call_data);
-			SetConnectID (connect_id, IS_CLOSED);
-		}
-	
-		DisplayMainWindow();
+    IMChangeFocusStruct call_data;
+
+    if (!CurrentIC)
+	return;
+
+    if (connect_id && CurrentIC->id) {
+	call_data.connect_id = connect_id;
+	call_data.icid = CurrentIC->id;
+
+	if (bState) {
+	    IMPreeditStart (ims, (XPointer) & call_data);
+	    SetConnectID (connect_id, IS_CHN);
 	}
+	else {
+	    IMPreeditEnd (ims, (XPointer) & call_data);
+	    SetConnectID (connect_id, IS_CLOSED);
+	    bVK = False;
+	    XUnmapWindow (dpy, inputWindow);
+	    XUnmapWindow (dpy, VKWindow);
+
+	    SwitchIM (-2);
+	}
+
+	DisplayMainWindow ();
+    }
 }
 
 void CreateConnectID (IMOpenStruct * call_data)
@@ -447,7 +556,7 @@ void CreateConnectID (IMOpenStruct * call_data)
     connectIDNew->next = (CONNECT_ID *) NULL;
     connectIDNew->connect_id = call_data->connect_id;
     connectIDNew->imState = IS_CLOSED;
-    connectIDNew->bReset = !bLumaQQ;
+    //connectIDNew->bReset = !bLumaQQ;
     connectIDNew->bTrackCursor = False;
     //connectIDNew->strLocale=(char *)malloc(sizeof(char)*(call_data->lang.length+1));
     //strcpy(connectIDNew->strLocale,call_data->lang.name);
@@ -472,7 +581,7 @@ void DestroyConnectID (CARD16 connect_id)
     }
 
     if (!temp)
-        return;
+	return;
 
     if (temp == connectIDsHead)
 	connectIDsHead = temp->next;
@@ -515,6 +624,7 @@ IME_STATE ConnectIDGetState (CARD16 connect_id)
     return IS_CLOSED;
 }
 
+/*New Lumaqq need not be supported specially
 Bool ConnectIDGetReset (CARD16 connect_id)
 {
     CONNECT_ID     *temp;
@@ -547,6 +657,18 @@ void ConnectIDSetReset (CARD16 connect_id, Bool bReset)
     }
 }
 
+void ConnectIDResetReset (void)
+{
+    CONNECT_ID     *temp;
+
+    temp = connectIDsHead;
+
+    while (temp) {
+	temp->bReset = True;
+	temp = temp->next;
+    }
+}
+*/
 Bool ConnectIDGetTrackCursor (CARD16 connect_id)
 {
     CONNECT_ID     *temp;
@@ -563,18 +685,6 @@ Bool ConnectIDGetTrackCursor (CARD16 connect_id)
     return False;
 }
 
-void ConnectIDResetReset (void)
-{
-    CONNECT_ID     *temp;
-
-    temp = connectIDsHead;
-
-    while (temp) {
-	temp->bReset = True;
-	temp = temp->next;
-    }
-}
-
 void ConnectIDSetTrackCursor (CARD16 connect_id, Bool bTrackCursor)
 {
     CONNECT_ID     *temp;
